# tests.test_features.test_feature_correlation
# Test the feature correlation visualizers
#
# Author:  Zijie (ZJ) Poh
# Created: Tue Jul 31 20:21:32 2018 -0700
#
# Copyright (C) 2018 The scikit-yb developers
# For license information, see LICENSE.txt
#
# ID: test_feature_correlation.py [33aec16] 8103276+zjpoh@users.noreply.github.com $

"""
Test the feature correlation to dependent variable visualizer.
"""

##########################################################################
## Imports
##########################################################################

import sys
import pytest
import numpy as np
import numpy.testing as npt


from yellowbrick.target import FeatureCorrelation, feature_correlation
from yellowbrick.exceptions import YellowbrickValueError, YellowbrickWarning

from sklearn import datasets
from tests.base import VisualTestCase

try:
    import pandas as pd
except ImportError:
    pd = None


##########################################################################
## Feature Correlation Tests
##########################################################################


class TestFeatureCorrelationVisualizer(VisualTestCase):
    """
    FeatureCorrelation visualizer
    """

    data = datasets.load_diabetes()
    X, y = data["data"], data["target"]
    labels = data["feature_names"]

    @pytest.mark.xfail(sys.platform == "win32", reason="images not close on windows")
    def test_feature_correlation_integrated_pearson(self):
        """
        Test FeatureCorrelation visualizer with pearson correlation
        coefficient
        """
        viz = FeatureCorrelation()
        viz.fit(self.X, self.y)
        viz.finalize()

        self.assert_images_similar(viz)

    @pytest.mark.xfail(sys.platform == "win32", reason="images not close on windows")
    def test_feature_correlation_integrated_mutual_info_regression(self):
        """
        Test FeatureCorrelation visualizer with mutual information regression
        """
        viz = FeatureCorrelation(method="mutual_info-regression")
        viz.fit(self.X, self.y, random_state=23456)
        viz.finalize()

        self.assert_images_similar(viz)

    @pytest.mark.xfail(sys.platform == "win32", reason="images not close on windows")
    def test_feature_correlation_integrated_mutual_info_classification(self):
        """
        Test FeatureCorrelation visualizer with mutual information
        on wine dataset (classification)
        """
        data = datasets.load_wine()
        X, y = data["data"], data["target"]

        viz = FeatureCorrelation(method="mutual_info-classification")
        viz.fit(X, y, random_state=12345)
        viz.finalize()

        self.assert_images_similar(viz)

    def test_feature_correlation_method_not_implemented(self):
        """
        Test FeatureCorrelation visualizer with unknown method
        """
        method = "foo"
        e = "Method foo not implement; choose from *"
        with pytest.raises(YellowbrickValueError, match=e):
            FeatureCorrelation(method=method)

    def test_feature_correlation_labels_from_index(self):
        """
        Test getting feature labels from index
        """
        viz = FeatureCorrelation()
        viz.fit(self.X, self.y)

        npt.assert_array_equal(viz.features_, np.arange(self.X.shape[1]))

    def test_feature_correlation_labels(self):
        """
        Test labels as feature labels
        """
        viz = FeatureCorrelation(labels=self.labels)
        viz.fit(self.X, self.y)

        npt.assert_array_equal(viz.features_, self.labels)

    @pytest.mark.skipif(pd is None, reason="requires pandas")
    def test_feature_correlation_labels_from_dataframe(self):
        """
        Test getting feature labels from DataFrame
        """
        X_pd = pd.DataFrame(self.X, columns=self.labels)

        viz = FeatureCorrelation()
        viz.fit(X_pd, self.y)

        npt.assert_array_equal(viz.features_, self.labels)

    def test_feature_correlation_select_feature_by_index_out_of_range(self):
        """
        Test selecting feature by feature index but index is out of range
        """
        e = "Feature index is out of range"
        with pytest.raises(YellowbrickValueError, match=e):
            viz = FeatureCorrelation(feature_index=[0, 2, 10])
            viz.fit(self.X, self.y)

    def test_feature_correlation_select_feature_by_index(self):
        """
        Test selecting feature by index
        """
        viz = FeatureCorrelation(feature_index=[0, 2, 3])
        viz.fit(self.X, self.y)

        assert viz.scores_.shape[0] == 3

    def test_feature_correlation_select_feature_by_index_and_name(self):
        """
        Test selecting feature warning when both index and names are provided
        """
        feature_index = [0, 2, 3]
        feature_names = ["age"]

        e = (
            "Both feature_index and feature_names are specified. "
            "feature_names is ignored"
        )
        with pytest.raises(YellowbrickWarning, match=e):
            viz = FeatureCorrelation(
                feature_index=feature_index, feature_names=feature_names
            )
            viz.fit(self.X, self.y)
            assert viz.scores_.shape[0] == 3

    def test_feature_correlation_select_feature_by_name_no_labels(self):
        """
        Test selecting feature by feature names with labels is not supplied
        """
        feature_names = ["age"]

        e = "age not in labels"
        with pytest.raises(YellowbrickValueError, match=e):
            viz = FeatureCorrelation(feature_names=feature_names)
            viz.fit(self.X, self.y)

    def test_feature_correlation_select_feature_by_name(self):
        """
        Test selecting feature by feature names
        """
        feature_names = ["age", "sex", "bp", "s5"]

        viz = FeatureCorrelation(labels=self.labels, feature_names=feature_names)
        viz.fit(self.X, self.y)

        npt.assert_array_equal(viz.features_, feature_names)

    def test_feature_correlation_sort(self):
        """
        Test sorting of correlation
        """
        viz = FeatureCorrelation(sort=True)
        viz.fit(self.X, self.y)

        assert np.all(viz.scores_[:-1] <= viz.scores_[1:])

    @pytest.mark.xfail(sys.platform == "win32", reason="images not close on windows")
    def test_feature_correlation_quick_method(self):
        """
        Test sorting of correlation
        """
        g = feature_correlation.feature_correlation(self.X, self.y, labels=self.labels, show=False)

        self.assert_images_similar(g)



# tests.test_utils.test_decorators
# Tests for the decorators module in Yellowbrick utils.
#
# Author:   Benjamin Bengfort <bbengfort@districtdatalabs.com>
# Created:  Thu May 18 15:14:34 2017 -0400
#
# Copyright (C) 2017 The scikit-yb developers
# For license information, see LICENSE.txt
#
# ID: test_decorators.py [79cd8cf] benjamin@bengfort.com $

"""
Tests for the decorators module in Yellowbrick utils.
"""

##########################################################################
## Imports
##########################################################################

from yellowbrick.utils.decorators import *


##########################################################################
## Decorator Tests
##########################################################################


class TestDecorators(object):
    """
    Tests for the decorator utilities.
    """

    def test_memoization(self):
        """
        Test the memoized property decorator on a class.
        """

        class Visualizer(object):
            @memoized
            def foo(self):
                return "bar"

        viz = Visualizer()
        assert not hasattr(viz, "_foo")
        assert viz.foo == "bar"
        assert viz._foo == "bar"

    def test_docutil(self):
        """
        Test the docutil docstring copying methodology.
        """

        class Visualizer(object):
            def __init__(self):
                """
                This is the correct docstring.
                """
                pass

        def undecorated(*args, **kwargs):
            """
            This is an undecorated function string.
            """
            pass

        # Test the undecorated string to protect from magic
        assert undecorated.__doc__.strip() == "This is an undecorated function string."

        # Decorate manually and test the newly decorated return function.
        decorated = docutil(Visualizer.__init__)(undecorated)
        assert decorated.__doc__.strip() == "This is the correct docstring."

        # Assert that decoration modifies the original function.
        assert undecorated.__doc__.strip() == "This is the correct docstring."

        @docutil(Visualizer.__init__)
        def sugar(*args, **kwargs):
            pass

        # Assert that syntactic sugar works as expected.
        assert sugar.__doc__.strip() == "This is the correct docstring."



# tests.test_features.test_base
# Tests for the feature selection and analysis base classes
#
# Author:   Benjamin Bengfort
# Created:  Fri Oct 07 13:43:55 2016 -0400
#
# Copyright (C) 2016 The scikit-yb developers
# For license information, see LICENSE.txt
#
# ID: test_base.py [2e898a6] benjamin@bengfort.com $

"""
Tests for the feature selection and analysis base classes
"""

##########################################################################
## Imports
##########################################################################

import pytest
import numpy as np
import numpy.testing as npt

from tests.fixtures import Dataset
from yellowbrick.base import Visualizer
from yellowbrick.features.base import *

from sklearn.base import BaseEstimator, TransformerMixin
from sklearn.datasets import make_classification, make_regression

from unittest.mock import Mock

try:
    import pandas as pd
except ImportError:
    pd = None


##########################################################################
## Fixtures
##########################################################################


@pytest.fixture(scope="class")
def discrete(request):
    """
    Create a random classification dataset fixture.
    """
    X, y = make_classification(
        n_classes=5,
        n_samples=400,
        n_features=12,
        n_informative=10,
        n_redundant=0,
        random_state=2019,
    )

    # Dataset is accessible on the class so it is only generated once
    request.cls.discrete = Dataset(X, y)


@pytest.fixture(scope="class")
def continuous(request):
    """
    Creates a random regression dataset fixture.
    """
    X, y = make_regression(
        n_samples=500, n_features=22, n_informative=8, random_state=2019
    )

    # Dataset is accessible on the class so it is only generated once
    request.cls.continuous = Dataset(X, y)


##########################################################################
## FeatureVisualizer Base Tests
##########################################################################


@pytest.mark.usefixtures("discrete")
class TestFeatureVisualizer(object):
    """
    Test FeatureVisualizer base class
    """

    def test_subclass(self):
        """
        Check the visualizer/transformer class hierarchy
        """
        visualizer = FeatureVisualizer()
        assert isinstance(visualizer, TransformerMixin)
        assert isinstance(visualizer, BaseEstimator)
        assert isinstance(visualizer, Visualizer)

    def test_transform_returns_unmodified(self):
        """
        Ensure transformer is just a passthrough
        """
        X, y = self.discrete
        visualizer = FeatureVisualizer().fit(X, y)
        assert visualizer.transform(X, y) is X

    def test_fit_transform_show(self):
        """
        Test the fit/transform/show quick method
        """

        class MockFeatureVisaulizer(FeatureVisualizer):
            pass

        viz = MockFeatureVisaulizer()
        viz.fit = Mock(return_value=viz)
        viz.transform = Mock(return_value="a")
        viz.show = Mock()

        X, y = self.discrete
        assert viz.fit_transform_show(X, y, outpath="a.png", clear_figure=True) == "a"
        # Check that methods were called once (numpy arrays can't be compared with ==)
        assert viz.fit.call_count == 1
        assert viz.transform.call_count == 1
        viz.show.assert_called_once_with(outpath="a.png", clear_figure=True)


##########################################################################
## MultiFeatureVisualizer Tests
##########################################################################


@pytest.mark.usefixtures("discrete")
class TestMultiFeatureVisualizer(object):
    """
    Test the MultiFeatureVisualizer base class
    """

    def test_subclass(self):
        """
        Check the visualizer/transformer class hierarchy
        """
        visualizer = MultiFeatureVisualizer()
        assert isinstance(visualizer, FeatureVisualizer)
        assert isinstance(visualizer, TransformerMixin)
        assert isinstance(visualizer, BaseEstimator)
        assert isinstance(visualizer, Visualizer)

    def test_user_supplied_features(self):
        """
        Test that a user can supply feature names directly
        """
        X, y = self.discrete
        features = ["f{}".format(i + 1) for i in range(X.shape[1])]
        oz = MultiFeatureVisualizer(features=features)

        assert not hasattr(oz, "features_")
        assert oz.fit(X, y) is oz
        assert hasattr(oz, "features_")
        npt.assert_array_equal(oz.features_, np.asarray(features))

    def test_numeric_features(self):
        """
        Test that the features are column indices for numpy arrays
        """
        X, y = self.discrete
        oz = MultiFeatureVisualizer()

        assert not hasattr(oz, "features_")
        assert oz.fit(X, y) is oz
        assert hasattr(oz, "features_")
        assert len(oz.features_) == X.shape[1]

    @pytest.mark.skipif(pd is None, reason="test requires pandas")
    def test_pandas_string_columns(self):
        """
        Ensure that DataFrame column names are uses as features
        """
        X, y = self.discrete
        features = ["f{}".format(i + 1) for i in range(X.shape[1])]

        X = pd.DataFrame(X, columns=features)
        y = pd.Series(y, name="target")

        oz = MultiFeatureVisualizer()
        assert not hasattr(oz, "features_")
        assert oz.fit(X, y) is oz
        assert hasattr(oz, "features_")
        npt.assert_array_equal(oz.features_, np.asarray(features))

    @pytest.mark.skipif(pd is None, reason="test requires pandas")
    def test_pandas_no_columns(self):
        """
        Test when a DataFrame has no column names
        """
        """
        Ensure that Pandas column names are uses as features
        """
        X, y = self.discrete

        X = pd.DataFrame(X)
        y = pd.Series(y, name="target")

        oz = MultiFeatureVisualizer()
        assert not hasattr(oz, "features_")
        assert oz.fit(X, y) is oz
        assert hasattr(oz, "features_")
        assert len(oz.features_) == X.shape[1]


##########################################################################
## DataVisualizer Tests
##########################################################################


@pytest.mark.usefixtures("discrete", "continuous")
class TestDataVisualizer(object):
    """
    Test DataVisualizer base class
    """

    FIELDS = (
        "features_",
        "classes_",
        "range_",
        "_colors",
        "_target_color_type",
        "_label_encoder",
    )

    def assert_not_fitted(self, obj):
        __tracebackhide__ = True
        for field in self.FIELDS:
            if field.endswith("_"):
                assert not hasattr(obj, field), "has {} before fit".format(field)
            else:
                msg = "missing internal var {}".format(field)
                assert getattr(obj, field) is None, msg

    def assert_fitted(self, obj, fields=FIELDS):
        __tracebackhide__ = True
        # Mutually exclusive fields
        if obj._target_color_type == TargetType.SINGLE:
            assert (not hasattr(obj, "classes_")) and (not hasattr(obj, "range_"))

        elif obj._target_color_type == TargetType.DISCRETE:
            assert hasattr(obj, "_label_encoder")
            assert hasattr(obj, "classes_") and (not hasattr(obj, "range_"))

        elif obj._target_color_type == TargetType.CONTINUOUS:
            assert (not hasattr(obj, "classes_")) and hasattr(obj, "range_")

        else:
            raise ValueError(
                "cannot test target type {}".format(obj._target_color_type)
            )

        for field in fields:
            if field in {"classes_", "range_", "_label_encoder"}:
                continue  # handled by mutually exclusive
            assert hasattr(obj, field)

    def test_single_when_none(self):
        """
        Ensure that the target type is "single" when y is None
        """
        X, _ = self.discrete
        oz = DataVisualizer()
        assert oz.target_type == TargetType.AUTO

        # Assert single when y is None
        self.assert_not_fitted(oz)
        assert oz.fit(X, y=None) is oz
        self.assert_fitted(oz)
        assert oz._target_color_type == TargetType.SINGLE

    @pytest.mark.parametrize("target_type", ("discrete", "continuous"))
    def test_none_overrides_user_specified_target(self, target_type):
        """
        Even if a user supplies a target type it should be overriden by y=None
        """
        X, _ = getattr(self, target_type)
        oz = DataVisualizer(target_type=target_type).fit(X, y=None)

        assert oz._colors == "C0"
        assert oz._target_color_type == TargetType.SINGLE

    @pytest.mark.parametrize("dataset", ("discrete", "continuous"))
    def test_user_overrides_auto_target(self, dataset):
        """
        Ensure user specified target type overrides auto discovery
        """
        X, y = getattr(self, dataset)
        target_type = (
            TargetType.CONTINUOUS if dataset == "discrete" else TargetType.DISCRETE
        )

        oz = DataVisualizer(target_type=target_type)
        assert oz.target_type != TargetType.AUTO
        oz.fit(X, y)
        assert oz.target_type == target_type

    def test_continuous(self):
        """
        Test data visualizer on continuous data
        """
        # Check when y is continuous
        X, y = self.continuous
        oz = DataVisualizer()
        assert oz.target_type == TargetType.AUTO

        self.assert_not_fitted(oz)
        assert oz.fit(X, y) is oz
        self.assert_fitted(oz)
        assert oz._target_color_type == TargetType.CONTINUOUS
        assert oz.range_ == (y.min(), y.max())

    def test_discrete(self):
        """
        Test data visualizer on discrete data
        """
        # Check when y is discrete
        X, y = self.discrete
        oz = DataVisualizer()
        assert oz.target_type == TargetType.AUTO

        self.assert_not_fitted(oz)
        assert oz.fit(X, y) is oz
        self.assert_fitted(oz)
        assert oz._target_color_type == TargetType.DISCRETE
        assert len(oz.classes_) == np.unique(y).shape[0]

    def test_bad_target_type(self):
        """
        Assert target type is validated on init
        """
        msg = "unknown target color type 'foo'"
        with pytest.raises(YellowbrickValueError, match=msg):
            DataVisualizer(target_type="foo")

    def test_classes_discrete(self):
        """
        Ensure classes are assigned correctly for label encoding
        """
        X, y = self.discrete
        classes = ["a", "b", "c", "d", "e"]
        oz = DataVisualizer(classes=classes, target_type="discrete").fit(X, y)

        npt.assert_array_equal(oz.classes_, classes)
        assert list(oz._colors.keys()) == classes

    def test_classes_continuous(self):
        """
        Ensure classes are ignored in continuous case
        """
        X, y = self.continuous
        classes = ["a", "b", "c", "d", "e"]
        oz = DataVisualizer(classes=classes, target_type="continuous").fit(X, y)

        assert not hasattr(oz, "classes_")

    def test_get_target_color_type(self):
        """
        Test the get_target_color_type helper method
        """
        oz = DataVisualizer()

        with pytest.raises(NotFitted, match="unknown target color type"):
            oz.get_target_color_type()

        oz.fit(*self.continuous)
        assert oz.get_target_color_type() == TargetType.CONTINUOUS

    def test_get_colors_not_fitted(self):
        """
        Assert get_colors requires an fitted visualizer
        """
        oz = DataVisualizer()
        with pytest.raises(NotFitted, match="cannot determine colors"):
            oz.get_colors(["a", "b", "c"])

    @pytest.mark.parametrize("color, expected", [(None, "C0"), ("#F3B8AB", "#F3B8AB")])
    def test_get_colors_single(self, color, expected):
        """
        Test color assignment for single target type
        """
        X, y = self.discrete
        oz = DataVisualizer(colors=color).fit(X)
        assert oz.get_target_color_type() == TargetType.SINGLE

        # Test default colors
        colors = oz.get_colors(y)
        assert len(colors) == len(y)
        assert np.unique(colors) == expected

    def test_get_colors_discrete(self):
        """
        Test discrete colors with no label encoding
        """
        X, y = self.discrete
        oz = DataVisualizer().fit(X, y)
        assert oz.get_target_color_type() == TargetType.DISCRETE

        colors = oz.get_colors(y)
        assert len(colors) == len(y)
        assert set(colors) == set(oz._colors.values())

    def test_get_colors_discrete_classes(self):
        """
        Test discrete colors with label encoding and colors
        """
        X, y = self.discrete
        oz = DataVisualizer(
            classes=["a", "b", "c", "d", "e"], colors=["g", "r", "b", "m", "y"]
        ).fit(X, y)
        assert oz.get_target_color_type() == TargetType.DISCRETE

        colors = oz.get_colors(y)
        assert len(colors) == len(y)
        assert set(colors) == set(["g", "r", "b", "m", "y"])

    def test_get_colors_not_label_encoded(self):
        """
        Assert exception is raised on unknown class label for get_colors
        """
        X, y = self.discrete
        oz = DataVisualizer(classes="abcde").fit(X, y)

        with pytest.raises(YellowbrickKeyError, match="could not determine color"):
            oz.get_colors(["foo"])

    @pytest.mark.parametrize(
        "colors, colormap",
        [
            (["#3f78de", "#f38b33"], None),
            (["b", "g", "r", "m", "y"], None),
            (None, "Blues"),
        ],
    )
    def test_user_get_colors_discrete(self, colors, colormap):
        """
        Test the ways that users can specify colors
        """
        X, y = self.discrete
        oz = DataVisualizer(
            colors=colors, colormap=colormap, target_type="discrete"
        ).fit(X, y)

        colors = oz.get_colors(y)
        assert len(colors) == len(y)

    def test_get_colors_continous(self):
        """
        Test continuous colors with no default colormap
        """
        X, y = self.continuous
        oz = DataVisualizer().fit(X, y)
        assert oz.get_target_color_type() == TargetType.CONTINUOUS

        colors = oz.get_colors(y)
        assert len(colors) == len(y)

    def test_get_colors_continous_cmap(self):
        """
        Test continuous colors with user specified cmap
        """
        X, y = self.continuous
        oz = DataVisualizer(colormap="jet").fit(X, y)
        assert oz.get_target_color_type() == TargetType.CONTINUOUS

        colors = oz.get_colors(y)
        assert len(colors) == len(y)



# tests.test_classifier.test_prcurve
# Tests for the Precision-Recall curves visualizer
#
# Author:  Benjamin Bengfort
# Created: Tue Sep 04 16:48:09 2018 -0400
#
# Copyright (C) 2018 The scikit-yb developers
# For license information, see LICENSE.txt
#
# ID: test_prcurve.py [48889c4] benjamin@bengfort.com $

"""
Tests for the Precision-Recall curves visualizer
"""

##########################################################################
## Imports
##########################################################################

import sys
import pytest
import matplotlib

from yellowbrick.exceptions import *
from yellowbrick.classifier.prcurve import *
from yellowbrick.datasets import load_occupancy

from tests.base import IS_WINDOWS_OR_CONDA, VisualTestCase
from .test_rocauc import FakeClassifier

from sklearn.svm import LinearSVC, SVC
from sklearn.naive_bayes import GaussianNB
from sklearn.datasets import make_regression
from sklearn.tree import DecisionTreeClassifier
from sklearn.linear_model import RidgeClassifier
from sklearn.model_selection import train_test_split as tts
from sklearn.ensemble import RandomForestClassifier, AdaBoostClassifier
from sklearn.pipeline import Pipeline
from sklearn.preprocessing import MinMaxScaler

try:
    import pandas as pd
except ImportError:
    pd = None

##########################################################################
## Assertion Helpers
##########################################################################

LEARNED_FIELDS = ("target_type_", "score_", "precision_", "recall_")


def assert_not_fitted(oz):
    for field in LEARNED_FIELDS:
        assert not hasattr(oz, field)


def assert_fitted(oz):
    for field in LEARNED_FIELDS:
        assert hasattr(oz, field)


##########################################################################
## PrecisionRecallCurve Tests
##########################################################################


@pytest.mark.usefixtures("binary", "multiclass")
class TestPrecisionRecallCurve(VisualTestCase):
    """
    Test the PrecisionRecallCurve visualizer
    """

    def test_fit_continuous(self):
        """
        Should not allow any target type other than binary or multiclass
        """
        X, y = make_regression()
        with pytest.raises(YellowbrickValueError, match="does not support target type"):
            oz = PrecisionRecallCurve(LinearSVC())
            oz.fit(X, y)

    def test_ensure_fit(self):
        """
        Requires visualizer to be fit
        """
        with pytest.raises(
            NotFitted, match="this PrecisionRecallCurve instance is not fitted yet"
        ):
            oz = PrecisionRecallCurve(RidgeClassifier())
            oz.score(self.binary.X.test, self.binary.y.test)

    def test_binary_probability(self):
        """
        Visual similarity of binary classifier with predict_proba function
        """
        # Create and fit the visualizer
        oz = PrecisionRecallCurve(RandomForestClassifier(random_state=12))
        assert_not_fitted(oz)

        # Fit returns self
        assert oz.fit(self.binary.X.train, self.binary.y.train) is oz

        # Score the visualizer
        s = oz.score(self.binary.X.test, self.binary.y.test)
        assert_fitted(oz)

        # Score should be between 0 and 1
        assert 0.0 <= s <= 1.0

        # Check the binary classification properties
        assert oz.target_type_ == BINARY
        assert isinstance(oz.score_, float)
        assert oz.score_ == s
        assert isinstance(oz.precision_, np.ndarray)
        assert isinstance(oz.recall_, np.ndarray)

        # Compare the images
        oz.finalize()
        tol = (
            1.5 if sys.platform == "win32" else 1.0
        )  # fails with RMSE 1.409 on AppVeyor
        self.assert_images_similar(oz, tol=tol)

    @pytest.mark.xfail(
        IS_WINDOWS_OR_CONDA,
        reason="font rendering different in OS and/or Python; see #892",
    )
    def test_binary_probability_decision(self):
        """
        Visual similarity of binary classifier with both predict_proba & decision
        """
        # Create and fit the visualizer
        oz = PrecisionRecallCurve(AdaBoostClassifier(), iso_f1_curves=True)
        assert_not_fitted(oz)

        # Fit returns self
        assert oz.fit(self.binary.X.train, self.binary.y.train) is oz

        # Score the visualizer
        s = oz.score(self.binary.X.test, self.binary.y.test)
        assert_fitted(oz)

        # Score should be between 0 and 1
        assert 0.0 <= s <= 1.0

        # Check the binary classification properties
        assert oz.target_type_ == BINARY
        assert isinstance(oz.score_, float)
        assert oz.score_ == s
        assert isinstance(oz.precision_, np.ndarray)
        assert isinstance(oz.recall_, np.ndarray)

        # Compare the images
        oz.finalize()
        self.assert_images_similar(oz)

    def test_binary_decision(self):
        """
        Visual similarity of binary classifier with a decision function
        """
        # Create and fit the visualizer
        oz = PrecisionRecallCurve(LinearSVC(random_state=232))
        assert_not_fitted(oz)

        # Fit returns self
        assert oz.fit(self.binary.X.train, self.binary.y.train) is oz

        # Score the visualizer
        s = oz.score(self.binary.X.test, self.binary.y.test)
        assert_fitted(oz)

        # Score should be between 0 and 1
        assert 0.0 <= s <= 1.0

        # Check the binary classification properties
        assert oz.target_type_ == BINARY
        assert isinstance(oz.score_, float)
        assert oz.score_ == s
        assert isinstance(oz.precision_, np.ndarray)
        assert isinstance(oz.recall_, np.ndarray)

        # Compare the images
        # NOTE: do not finalize image to ensure tests pass on Travis
        # Fails with 3.083 on Travis-CI (passes on AppVeyor)
        self.assert_images_similar(oz, tol=3.5)

    def test_multiclass_decision(self):
        """
        Visual similarity of multiclass classifier with a decision function
        """
        # Create and fit the visualizer
        oz = PrecisionRecallCurve(RidgeClassifier(random_state=993))
        assert_not_fitted(oz)

        # Fit returns self
        assert oz.fit(self.multiclass.X.train, self.multiclass.y.train) is oz

        # Score the visualizer
        s = oz.score(self.multiclass.X.test, self.multiclass.y.test)
        assert_fitted(oz)

        # Score should be between 0 and 1
        assert 0.0 <= s <= 1.0

        # Check the multiclass classification properties
        assert oz.target_type_ == MULTICLASS
        assert isinstance(oz.score_, dict)
        assert oz.score_[MICRO] == s
        assert isinstance(oz.precision_, dict)
        assert isinstance(oz.recall_, dict)
        assert len(oz.score_) == len(oz.classes_) + 1
        assert len(oz.precision_) == len(oz.classes_) + 1
        assert len(oz.recall_) == len(oz.classes_) + 1

        # Compare the images
        oz.finalize()
        tol = (
            1.25 if sys.platform == "win32" else 1.0
        )  # fails with RMSE 1.118 on AppVeyor
        self.assert_images_similar(oz, tol=tol)

    @pytest.mark.xfail(
        IS_WINDOWS_OR_CONDA,
        reason="font rendering different in OS and/or Python; see #892",
    )
    def test_multiclass_probability(self):
        """
        Visual similarity of multiclass classifier with predict_proba function
        """
        # Create and fit the visualizer
        oz = PrecisionRecallCurve(
            GaussianNB(),
            per_class=True,
            micro=False,
            fill_area=False,
            iso_f1_curves=True,
            ap_score=False,
        )
        assert_not_fitted(oz)

        # Fit returns self
        assert oz.fit(self.multiclass.X.train, self.multiclass.y.train) is oz

        # Score the visualizer
        s = oz.score(self.multiclass.X.test, self.multiclass.y.test)
        assert_fitted(oz)

        # Score should be between 0 and 1
        assert 0.0 <= s <= 1.0

        # Check the multiclass classification properties
        assert oz.target_type_ == MULTICLASS
        assert isinstance(oz.score_, dict)
        assert oz.score_[MICRO] == s
        assert isinstance(oz.precision_, dict)
        assert isinstance(oz.recall_, dict)
        assert len(oz.score_) == len(oz.classes_) + 1
        assert len(oz.precision_) == len(oz.classes_) + 1
        assert len(oz.recall_) == len(oz.classes_) + 1

        # Compare the images
        oz.finalize()
        self.assert_images_similar(oz)

    def test_multiclass_probability_with_class_labels(self):
        """Visual similarity of multiclass classifier with class labels."""
        # Create and fit the visualizer
        oz = PrecisionRecallCurve(
            GaussianNB(),
            per_class=True,
            micro=False,
            fill_area=False,
            iso_f1_curves=True,
            ap_score=False,
            classes=["a", "b", "c", "d", "e", "f"],
        )
        assert_not_fitted(oz)

        # Fit returns self
        assert oz.fit(self.multiclass.X.train, self.multiclass.y.train) is oz

        # Score the visualizer
        s = oz.score(self.multiclass.X.test, self.multiclass.y.test)
        assert_fitted(oz)

        # Score should be between 0 and 1
        assert 0.0 <= s <= 1.0

        # Check the multiclass classification properties
        assert oz.target_type_ == MULTICLASS
        assert isinstance(oz.score_, dict)
        assert oz.score_[MICRO] == s
        assert isinstance(oz.precision_, dict)
        assert isinstance(oz.recall_, dict)
        assert len(oz.score_) == len(oz.classes_) + 1
        assert len(oz.precision_) == len(oz.classes_) + 1
        assert len(oz.recall_) == len(oz.classes_) + 1

        # Finalize image
        oz.finalize()

        # Compare the label text of the images.
        assert oz.ax.get_xlabel() == "Recall"
        oz.ax.set_xlabel("")
        assert oz.ax.get_ylabel() == "Precision"
        oz.ax.set_ylabel("")
        assert oz.ax.get_title() == "Precision-Recall Curve for GaussianNB"
        oz.ax.set_title("")

        # Compare the Legend text
        expected_legend_txt = [
            "PR for class a (area=0.42)",
            "PR for class b (area=0.36)",
            "PR for class c (area=0.44)",
            "PR for class d (area=0.52)",
            "PR for class e (area=0.37)",
            "PR for class f (area=0.49)",
        ]
        assert [x.get_text() for x in oz.ax.legend().get_texts()] == expected_legend_txt
        oz.ax.get_legend().remove()

        # Text in iso_f1_curves.
        # Will not check for these as they appears okay in other test images.
        for child in oz.ax.get_children():
            if isinstance(child, matplotlib.text.Annotation):
                child.remove()

        # Compare the images
        tol = (
            6.6 if sys.platform == "win32" else 1.0
        )  # fails with RMSE 6.583 on AppVeyor
        self.assert_images_similar(oz, tol=tol)

    @pytest.mark.filterwarnings("ignore:From version 0.21")
    @pytest.mark.xfail(
        IS_WINDOWS_OR_CONDA,
        reason="font rendering different in OS and/or Python; see #892",
    )
    def test_quick_method(self):
        """
        Test the precision_recall_curve quick method with numpy arrays.
        """
        X, y = load_occupancy(return_dataset=True).to_numpy()
        model = DecisionTreeClassifier(random_state=23, max_depth=2)

        oz = precision_recall_curve(
            model,
            X,
            y,
            per_class=False,
            micro=True,
            fill_area=False,
            iso_f1_curves=True,
            ap_score=False,
            show=False,
        )

        assert isinstance(oz, PrecisionRecallCurve)
        self.assert_images_similar(oz)

    @pytest.mark.skipif(pd is None, reason="test requires pandas")
    def test_pandas_integration(self):
        """
        Test the precision_recall_curve with Pandas dataframes
        """
        X, y = load_occupancy(return_dataset=True).to_pandas()

        model = DecisionTreeClassifier(random_state=14)

        X_train, X_test, y_train, y_test = tts(
            X, y, test_size=0.2, shuffle=True, random_state=555
        )

        oz = PrecisionRecallCurve(
            model,
            per_class=True,
            micro=False,
            fill_area=False,
            iso_f1_curves=True,
            ap_score=False,
            classes=["unoccupied", "occupied"],
        )
        oz.fit(X_train, y_train)
        oz.score(X_test, y_test)

        oz.finalize()

        # Miniconda & Appveyor: images not close (RMS 5.089)
        self.assert_images_similar(oz, tol=5.5)

    def test_no_scoring_function(self):
        """
        Test get y scores with classifiers that have no scoring method
        """
        oz = PrecisionRecallCurve(FakeClassifier())
        with pytest.raises(
            ModelError, match="requires .* predict_proba or decision_function"
        ):
            oz._get_y_scores(self.binary.X.train)

    @pytest.mark.xfail(
        IS_WINDOWS_OR_CONDA,
        reason="font rendering different in OS and/or Python; see #892",
    )
    def test_custom_iso_f1_scores(self):
        """
        Test using custom ISO F1 Values
        """
        X, y = load_occupancy(return_dataset=True).to_numpy()

        vals = (0.1, 0.6, 0.3, 0.9, 0.9)
        viz = PrecisionRecallCurve(
            RandomForestClassifier(random_state=27),
            iso_f1_curves=True,
            iso_f1_values=vals,
        )

        X_train, X_test, y_train, y_test = tts(
            X, y, test_size=0.2, shuffle=True, random_state=555
        )

        assert viz.fit(X_train, y_train) is viz
        viz.score(X_test, y_test)
        viz.finalize()

        self.assert_images_similar(viz)

    def test_quick_method_with_test_set(self):
        """
        Test quick method when both train and test data is supplied
        """
        X, y = load_occupancy(return_dataset=True).to_numpy()

        X_train, X_test, y_train, y_test = tts(
            X, y, test_size=0.2, shuffle=True, random_state=555
        )

        viz = precision_recall_curve(
            RandomForestClassifier(random_state=72),
            X_train,
            y_train,
            X_test,
            y_test,
            show=False,
        )
        self.assert_images_similar(viz)

    def test_missing_test_data_in_quick_method(self):
        """
        Test quick method when test data is missing.
        """
        X, y = load_occupancy(return_dataset=True).to_numpy()

        X_train, X_test, y_train, y_test = tts(
            X, y, test_size=0.2, shuffle=True, random_state=55555
        )

        emsg = "both X_test and y_test are required if one is specified"

        with pytest.raises(YellowbrickValueError, match=emsg):
            precision_recall_curve(
                RandomForestClassifier(), X_train, y_train, y_test=y_test, show=False
            )

        with pytest.raises(YellowbrickValueError, match=emsg):
            precision_recall_curve(
                RandomForestClassifier(), X_train, y_train, X_test, show=False
            )

    def test_per_class_and_micro(self):
        """
        Test if both per_class and micro set to True, user gets micro ignored warning
        """
        msg = (
            "micro=True is ignored;"
            "specify per_class=False to draw a PR curve after micro-averaging"
        )
        with pytest.warns(YellowbrickWarning, match=msg):
            PrecisionRecallCurve(
                RidgeClassifier(random_state=13), micro=True, per_class=True, show=False
            )

    def test_within_pipeline(self):
        """
        Test that visualizer can be accessed within a sklearn pipeline
        """
        X, y = load_occupancy(return_dataset=True).to_pandas()
        classes = ["unoccupied", "occupied"]

        X_train, X_test, y_train, y_test = tts(
            X, y, test_size=0.2, shuffle=True, random_state=42
        )

        model = Pipeline(
            [
                ("minmax", MinMaxScaler()),
                (
                    "prc",
                    PrecisionRecallCurve(
                        SVC(random_state=42),
                        per_class=True,
                        micro=False,
                        fill_area=False,
                        iso_f1_curves=True,
                        ap_score=False,
                        classes=classes,
                    ),
                ),
            ]
        )

        model.fit(X_train, y_train)
        model.score(X_test, y_test)
        model["prc"].finalize()
        self.assert_images_similar(model["prc"], tol=5.5)

    def test_within_pipeline_quickmethod(self):
        """
        Test that visualizer quickmethod can be accessed within a
        sklearn pipeline
        """
        X, y = load_occupancy(return_dataset=True).to_pandas()

        X_train, X_test, y_train, y_test = tts(
            X, y, test_size=0.2, shuffle=True, random_state=42
        )

        model = Pipeline(
            [
                ("minmax", MinMaxScaler()),
                (
                    "prc",
                    precision_recall_curve(
                        SVC(random_state=42),
                        X_train,
                        y_train,
                        X_test,
                        y_test,
                        per_class=True,
                        micro=False,
                        fill_area=False,
                        iso_f1_curves=True,
                        ap_score=False,
                        classes=["unoccupied", "occupied"],
                        show=False,
                    ),
                ),
            ]
        )
        self.assert_images_similar(model["prc"], tol=5.5)

    def test_pipeline_as_model_input(self):
        """
        Test that visualizer can handle sklearn pipeline as model input
        """
        X, y = load_occupancy(return_dataset=True).to_pandas()
        classes = ["unoccupied", "occupied"]

        X_train, X_test, y_train, y_test = tts(
            X, y, test_size=0.2, shuffle=True, random_state=42
        )

        model = Pipeline([("minmax", MinMaxScaler()), ("svc", SVC(random_state=42))])

        oz = PrecisionRecallCurve(
            model,
            per_class=True,
            micro=False,
            fill_area=False,
            iso_f1_curves=True,
            ap_score=False,
            classes=classes,
        )
        oz.fit(X_train, y_train)
        oz.score(X_test, y_test)
        oz.finalize()
        self.assert_images_similar(oz, tol=5.5)

    def test_pipeline_as_model_input_quickmethod(self):
        """
        Test that visualizer can handle sklearn pipeline as model input
        within a quickmethod
        """
        X, y = load_occupancy(return_dataset=True).to_pandas()

        X_train, X_test, y_train, y_test = tts(
            X, y, test_size=0.2, shuffle=True, random_state=42
        )

        model = Pipeline([("minmax", MinMaxScaler()), ("svc", SVC(random_state=42))])

        oz = precision_recall_curve(
            model,
            X_train,
            y_train,
            X_test,
            y_test,
            per_class=True,
            micro=False,
            fill_area=False,
            iso_f1_curves=True,
            ap_score=False,
            classes=["unoccupied", "occupied"],
            show=False,
        )
        self.assert_images_similar(oz, tol=5.5)



# tests.test_contrib.test_missing.test_dispersion
# Tests for the alpha selection visualizations.
#
# Author:  Nathan Danielsen <nathan.danielsen@gmail.com>
# Created:  Thu Mar 29 12:13:04 2018 -0500
#
# Copyright (C) 2018 The scikit-yb developers
# For license information, see LICENSE.txt
#
# ID: test_dispersion.py [1443e16] ndanielsen@users.noreply.github.com $

"""
Tests for the MissingValuesDispersion visualizations.
"""

##########################################################################
## Imports
##########################################################################

import os
import pytest

from sklearn.datasets import make_classification
from tests.base import VisualTestCase

from yellowbrick.contrib.missing.dispersion import *

try:
    import pandas as pd
except ImportError:
    pd = None


@pytest.fixture(scope="class")
def missing_dispersion_tolerance(request):
    request.cls.tol = 0.5 if os.name == "nt" else 0.01


##########################################################################
## Feature Importances Tests
##########################################################################


@pytest.mark.usefixtures("missing_dispersion_tolerance")
class TestMissingValuesDispersion(VisualTestCase):
    """
    MissingValuesDispersion visualizer
    """

    @pytest.mark.skipif(pd is None, reason="pandas is required")
    def test_missingvaluesdispersion_with_pandas(self):
        """
        Integration test of visualizer with pandas
        """
        X, y = make_classification(
            n_samples=400,
            n_features=20,
            n_informative=8,
            n_redundant=8,
            n_classes=2,
            n_clusters_per_class=4,
            random_state=854,
        )

        # add nan values to a range of values in the matrix
        X[X > 1.5] = np.nan

        X_ = pd.DataFrame(X)
        features = [str(n) for n in range(20)]
        viz = MissingValuesDispersion(features=features)
        viz.fit(X_)
        viz.finalize()

        self.assert_images_similar(viz, tol=self.tol)

    @pytest.mark.skipif(pd is None, reason="pandas is required")
    def test_missingvaluesdispersion_with_pandas_with_y_targets(self):
        """
        Integration test of visualizer with pandas with y targets
        """
        X, y = make_classification(
            n_samples=400,
            n_features=20,
            n_informative=8,
            n_redundant=8,
            n_classes=2,
            n_clusters_per_class=4,
            random_state=854,
        )

        # add nan values to a range of values in the matrix
        X[X > 1.5] = np.nan

        X_ = pd.DataFrame(X)
        features = [str(n) for n in range(20)]
        classes = ["Class A", "Class B"]
        viz = MissingValuesDispersion(features=features, classes=classes)
        viz.fit(X_, y=y)
        viz.finalize()

        self.assert_images_similar(viz, tol=self.tol)

    def test_missingvaluesdispersion_with_numpy(self):
        """
        Integration test of visualizer with numpy
        """
        X, y = make_classification(
            n_samples=400,
            n_features=20,
            n_informative=8,
            n_redundant=8,
            n_classes=2,
            n_clusters_per_class=4,
            random_state=852,
        )

        # add nan values to a range of values in the matrix
        X[X > 1.5] = np.nan

        features = [str(n) for n in range(20)]
        viz = MissingValuesDispersion(features=features)
        viz.fit(X)
        viz.finalize()

        self.assert_images_similar(viz, tol=self.tol)

    def test_missingvaluesdispersion_with_numpy_with_y_targets(self):
        """
        Integration test of visualizer with numpy with y targets
        """
        X, y = make_classification(
            n_samples=400,
            n_features=20,
            n_informative=8,
            n_redundant=8,
            n_classes=2,
            n_clusters_per_class=4,
            random_state=852,
        )

        # add nan values to a range of values in the matrix
        X[X > 1.5] = np.nan

        features = [str(n) for n in range(20)]
        classes = ["Class A", "Class B"]
        viz = MissingValuesDispersion(features=features, classes=classes)
        viz.fit(X, y=y)
        viz.finalize()

        self.assert_images_similar(viz, tol=self.tol)



# tests.test_classifier.test_confusion_matrix
# Tests for the confusion matrix visualizer
#
# Aithor:  Neal Humphrey
# Author:  Benjamin Bengfort
# Created: Tue May 03 11:05:11 2017 -0700
#
# Copyright (C) 2017 The scikit-yb developers
# For license information, see LICENSE.txt
#
# ID: test_confusion_matrix.py [] benjamin@bengfort.com $

"""
Tests for the confusion matrix visualizer
"""

##########################################################################
## Imports
##########################################################################

import sys
import pytest
import yellowbrick as yb
import numpy.testing as npt
import matplotlib.pyplot as plt

from yellowbrick.exceptions import ModelError
from yellowbrick.datasets import load_occupancy
from yellowbrick.classifier.confusion_matrix import *

from unittest.mock import patch
from tests.fixtures import Dataset, Split
from tests.base import IS_WINDOWS_OR_CONDA, VisualTestCase

from sklearn.svm import SVC
from sklearn.naive_bayes import GaussianNB
from sklearn.preprocessing import LabelEncoder
from sklearn.tree import DecisionTreeClassifier
from sklearn.linear_model import LogisticRegression
from sklearn.linear_model import PassiveAggressiveRegressor
from sklearn.model_selection import train_test_split as tts
from sklearn.datasets import load_digits, make_classification
from sklearn.preprocessing import MinMaxScaler
from sklearn.pipeline import Pipeline

try:
    import pandas as pd
except ImportError:
    pd = None

##########################################################################
## Fixtures
##########################################################################


@pytest.fixture(scope="class")
def digits(request):
    """
    Creates a fixture of train and test splits for the sklearn digits dataset
    For ease of use returns a Dataset named tuple composed of two Split tuples.
    """
    data = load_digits()
    X_train, X_test, y_train, y_test = tts(
        data.data, data.target, test_size=0.2, random_state=11
    )

    # Set a class attribute for digits
    request.cls.digits = Dataset(Split(X_train, X_test), Split(y_train, y_test))


##########################################################################
## Test Cases
##########################################################################


@pytest.mark.usefixtures("digits")
class TestConfusionMatrix(VisualTestCase):
    """
    Test ConfusionMatrix visualizer
    """

    @pytest.mark.xfail(sys.platform == "win32", reason="images not close on windows")
    def test_confusion_matrix(self):
        """
        Integration test on digits dataset with LogisticRegression
        """
        _, ax = plt.subplots()

        model = LogisticRegression(random_state=93)
        cm = ConfusionMatrix(model, ax=ax, classes=[0, 1, 2, 3, 4, 5, 6, 7, 8, 9])
        cm.fit(self.digits.X.train, self.digits.y.train)
        cm.score(self.digits.X.test, self.digits.y.test)

        self.assert_images_similar(cm, tol=10)

        # Ensure correct confusion matrix under the hood
        npt.assert_array_equal(
            cm.confusion_matrix_,
            np.array(
                [
                    [38, 0, 0, 0, 0, 0, 0, 0, 0, 0],
                    [0, 35, 0, 0, 0, 0, 0, 0, 2, 0],
                    [0, 0, 39, 0, 0, 0, 0, 0, 0, 0],
                    [0, 0, 0, 38, 0, 0, 0, 0, 3, 0],
                    [0, 0, 0, 0, 40, 0, 0, 1, 0, 0],
                    [0, 0, 0, 0, 0, 27, 0, 0, 0, 0],
                    [0, 0, 0, 0, 0, 1, 29, 0, 0, 0],
                    [0, 0, 0, 0, 0, 0, 0, 35, 0, 1],
                    [0, 0, 0, 0, 0, 1, 0, 1, 31, 1],
                    [0, 0, 0, 0, 0, 0, 0, 0, 1, 36],
                ]
            ),
        )

    @pytest.mark.xfail(sys.platform == "win32", reason="images not close on windows")
    def test_no_classes_provided(self):
        """
        Integration test on digits dataset with GaussianNB, no classes
        """
        _, ax = plt.subplots()

        model = GaussianNB()
        cm = ConfusionMatrix(model, ax=ax, classes=None)
        cm.fit(self.digits.X.train, self.digits.y.train)
        cm.score(self.digits.X.test, self.digits.y.test)

        self.assert_images_similar(cm, tol=10)

        # Ensure correct confusion matrix under the hood
        npt.assert_array_equal(
            cm.confusion_matrix_,
            np.array(
                [
                    [36, 0, 0, 0, 1, 0, 0, 1, 0, 0],
                    [0, 31, 0, 0, 0, 0, 0, 1, 3, 2],
                    [0, 1, 34, 0, 0, 0, 0, 0, 4, 0],
                    [0, 1, 0, 33, 0, 2, 0, 2, 3, 0],
                    [0, 0, 0, 0, 36, 0, 0, 5, 0, 0],
                    [0, 0, 0, 0, 0, 27, 0, 0, 0, 0],
                    [0, 0, 1, 0, 1, 0, 28, 0, 0, 0],
                    [0, 0, 0, 0, 0, 0, 0, 36, 0, 0],
                    [0, 3, 0, 1, 0, 1, 0, 4, 25, 0],
                    [1, 2, 0, 0, 1, 0, 0, 8, 3, 22],
                ]
            ),
        )

    def test_fontsize(self):
        """
        Test confusion matrix with smaller fontsize on digits dataset with SVC
        """
        _, ax = plt.subplots()

        model = SVC(random_state=93)
        cm = ConfusionMatrix(model, ax=ax, fontsize=8)

        cm.fit(self.digits.X.train, self.digits.y.train)
        cm.score(self.digits.X.test, self.digits.y.test)

        self.assert_images_similar(cm, tol=10)

    def test_percent_mode(self):
        """
        Test confusion matrix in percent mode on digits dataset with SVC
        """
        _, ax = plt.subplots()

        model = SVC(random_state=93)
        cm = ConfusionMatrix(model, ax=ax, percent=True)

        cm.fit(self.digits.X.train, self.digits.y.train)
        cm.score(self.digits.X.test, self.digits.y.test)

        self.assert_images_similar(cm, tol=10)

        # Ensure correct confusion matrix under the hood
        npt.assert_array_equal(
            cm.confusion_matrix_,
            np.array(
                [
                    [38, 0, 0, 0, 0, 0, 0, 0, 0, 0],
                    [0, 37, 0, 0, 0, 0, 0, 0, 0, 0],
                    [0, 0, 39, 0, 0, 0, 0, 0, 0, 0],
                    [0, 0, 0, 38, 0, 1, 0, 1, 1, 0],
                    [0, 0, 0, 0, 41, 0, 0, 0, 0, 0],
                    [0, 0, 0, 0, 0, 27, 0, 0, 0, 0],
                    [0, 0, 0, 0, 0, 0, 30, 0, 0, 0],
                    [0, 0, 0, 0, 0, 0, 0, 35, 0, 1],
                    [0, 0, 0, 0, 0, 0, 0, 0, 34, 0],
                    [0, 0, 0, 0, 0, 0, 0, 1, 0, 36],
                ]
            ),
        )

    @pytest.mark.xfail(reason="class filtering is not currently supported")
    def test_class_filter_eg_zoom_in(self):
        """
        Test filtering classes zooms in on the confusion matrix.
        """
        _, ax = plt.subplots()

        model = LogisticRegression(random_state=93)
        cm = ConfusionMatrix(model, ax=ax, classes=[0, 1, 2])
        cm.fit(self.digits.X.train, self.digits.y.train)
        cm.score(self.digits.X.test, self.digits.y.test)

        self.assert_images_similar(cm, tol=10)

        # Ensure correct confusion matrix under the hood
        npt.assert_array_equal(
            cm.confusion_matrix_, np.array([[38, 0, 0], [0, 35, 0], [0, 0, 39]])
        )

    def test_extra_classes(self):
        """
        Assert that any extra classes raise an exception
        """
        model = LogisticRegression(random_state=93)
        cm = ConfusionMatrix(model, classes=[0, 1, 2, 11])

        with pytest.raises(ModelError, match="could not decode"):
            cm.fit(self.digits.X.train, self.digits.y.train)

    def test_defined_mapping(self):
        """
        Test mapping as label encoder to define tick labels
        """
        _, ax = plt.subplots()

        model = LogisticRegression(random_state=93)
        classes = np.array(
            [
                "zero",
                "one",
                "two",
                "three",
                "four",
                "five",
                "six",
                "seven",
                "eight",
                "nine",
            ]
        )
        mapping = {
            0: "zero",
            1: "one",
            2: "two",
            3: "three",
            4: "four",
            5: "five",
            6: "six",
            7: "seven",
            8: "eight",
            9: "nine",
        }
        cm = ConfusionMatrix(model, ax=ax, encoder=mapping)
        cm.fit(self.digits.X.train, self.digits.y.train)
        cm.score(self.digits.X.test, self.digits.y.test)

        xlabels = np.array([l.get_text() for l in ax.get_xticklabels()])
        npt.assert_array_equal(xlabels, classes)

        ylabels = [l.get_text() for l in ax.get_yticklabels()]
        ylabels.reverse()
        ylabels = np.asarray(ylabels)
        npt.assert_array_equal(ylabels, classes)

    def test_inverse_mapping(self):
        """
        Test LabelEncoder as label encoder to define tick labels
        """
        _, ax = plt.subplots()

        model = LogisticRegression(random_state=93)
        le = LabelEncoder()
        le.fit(
            [
                "zero",
                "one",
                "two",
                "three",
                "four",
                "five",
                "six",
                "seven",
                "eight",
                "nine",
            ]
        )

        cm = ConfusionMatrix(model, ax=ax, encoder=le)
        cm.fit(self.digits.X.train, self.digits.y.train)
        cm.score(self.digits.X.test, self.digits.y.test)

        xlabels = np.array([l.get_text() for l in ax.get_xticklabels()])
        npt.assert_array_equal(xlabels, le.classes_)

        ylabels = [l.get_text() for l in ax.get_yticklabels()]
        ylabels.reverse()
        ylabels = np.asarray(ylabels)
        npt.assert_array_equal(ylabels, le.classes_)

    @pytest.mark.xfail(
        IS_WINDOWS_OR_CONDA,
        reason="font rendering different in OS and/or Python; see #892",
    )
    @pytest.mark.skipif(pd is None, reason="test requires pandas")
    def test_pandas_integration(self):
        """
        Test with Pandas DataFrame and Series input
        """
        _, ax = plt.subplots()

        # Load the occupancy dataset from fixtures
        X, y = load_occupancy(return_dataset=True).to_pandas()

        # Create train/test splits
        splits = tts(X, y, test_size=0.2, random_state=8873)
        X_train, X_test, y_train, y_test = splits

        # Create confusion matrix
        model = GaussianNB()
        cm = ConfusionMatrix(model, ax=ax, classes=None)
        cm.fit(X_train, y_train)
        cm.score(X_test, y_test)

        self.assert_images_similar(cm, tol=0.1)

        # Ensure correct confusion matrix under the hood
        npt.assert_array_equal(cm.confusion_matrix_, np.array([[3012, 114], [1, 985]]))

    @pytest.mark.xfail(
        IS_WINDOWS_OR_CONDA,
        reason="font rendering different in OS and/or Python; see #892",
    )
    def test_quick_method(self):
        """
        Test the quick method with a random dataset
        """
        X, y = make_classification(
            n_samples=400,
            n_features=20,
            n_informative=8,
            n_redundant=8,
            n_classes=2,
            n_clusters_per_class=4,
            random_state=27,
        )

        _, ax = plt.subplots()
        model = DecisionTreeClassifier(random_state=25)

        # compare the images
        visualizer = confusion_matrix(model, X, y, ax=ax, show=False)
        self.assert_images_similar(visualizer)

    def test_isclassifier(self):
        """
        Assert that only classifiers can be used with the visualizer.
        """
        model = PassiveAggressiveRegressor()
        message = (
            "This estimator is not a classifier; "
            "try a regression or clustering score visualizer instead!"
        )

        with pytest.raises(yb.exceptions.YellowbrickError, match=message):
            ConfusionMatrix(model)

    def test_score_returns_score(self):
        """
        Test that ConfusionMatrix score() returns a score between 0 and 1
        """
        # Load the occupancy dataset from fixtures
        X, y = load_occupancy(return_dataset=True).to_numpy()
        X_train, X_test, y_train, y_test = tts(X, y, test_size=0.2, random_state=42)

        # Create and fit the visualizer
        visualizer = ConfusionMatrix(LogisticRegression())
        visualizer.fit(X_train, y_train)

        # Score the visualizer
        s = visualizer.score(X_test, y_test)

        assert 0 <= s <= 1

    def test_with_fitted(self):
        """
        Test that visualizer properly handles an already-fitted model
        """
        X, y = load_occupancy(return_dataset=True).to_numpy()

        model = LogisticRegression().fit(X, y)
        classes = ["unoccupied", "occupied"]

        with patch.object(model, "fit") as mockfit:
            oz = ConfusionMatrix(model, classes=classes)
            oz.fit(X, y)
            mockfit.assert_not_called()

        with patch.object(model, "fit") as mockfit:
            oz = ConfusionMatrix(model, classes=classes, is_fitted=True)
            oz.fit(X, y)
            mockfit.assert_not_called()

        with patch.object(model, "fit") as mockfit:
            oz = ConfusionMatrix(model, classes=classes, is_fitted=False)
            oz.fit(X, y)
            mockfit.assert_called_once_with(X, y)

    
    def test_within_pipeline(self):
        """
        Test that visualizer can be accessed within a sklearn pipeline
        """
        X, y = load_occupancy(return_dataset=True).to_pandas()
        classes = ["unoccupied", "occupied"]

        X_train, X_test, y_train, y_test = tts(
                    X, y, test_size=0.2, shuffle=True, random_state=42
                )

        model = Pipeline([
            ('minmax', MinMaxScaler()), 
            ('matrix', ConfusionMatrix(SVC(random_state=42), classes=classes))
        ])

        model.fit(X_train, y_train)
        model.score(X_test, y_test)
        model['matrix'].finalize()
        self.assert_images_similar(model['matrix'], tol=12)

    def test_within_pipeline_quickmethod(self):
        """
        Test that visualizer quickmethod can be accessed within a
        sklearn pipeline
        """
        X, y = load_occupancy(return_dataset=True).to_pandas()

        X_train, X_test, y_train, y_test = tts(
                    X, y, test_size=0.2, shuffle=True, random_state=42
                )

        model = Pipeline([
            ('minmax', MinMaxScaler()), 
            ('matrix', confusion_matrix(SVC(random_state=42),
                                            X_train, y_train, X_test, y_test,
                                            classes=["vacant", "occupied"],
                                            show=False))
            ])
        self.assert_images_similar(model['matrix'], tol=12)

    def test_pipeline_as_model_input(self):
        """
        Test that visualizer can handle sklearn pipeline as model input
        """
        X, y = load_occupancy(return_dataset=True).to_pandas()
        classes = ["unoccupied", "occupied"]

        X_train, X_test, y_train, y_test = tts(
                    X, y, test_size=0.2, shuffle=True, random_state=42
                )

        model = Pipeline([
            ('minmax', MinMaxScaler()), 
            ('svc', SVC(random_state=42))
        ])

        oz = ConfusionMatrix(model, classes=classes)
        oz.fit(X_train, y_train)
        oz.score(X_test, y_test)
        oz.finalize()
        self.assert_images_similar(oz, tol=12)

    def test_pipeline_as_model_input_quickmethod(self):
        """
        Test that visualizer can handle sklearn pipeline as model input
        within a quickmethod
        """
        X, y = load_occupancy(return_dataset=True).to_pandas()

        X_train, X_test, y_train, y_test = tts(
                    X, y, test_size=0.2, shuffle=True, random_state=42
                )

        model = Pipeline([
            ('minmax', MinMaxScaler()), 
            ('svc', SVC(random_state=42))
        ])

        oz = confusion_matrix(model,
                                   X_train, y_train, X_test, y_test,
                                   classes=["vacant", "occupied"],
                                   show=False)
        self.assert_images_similar(oz, tol=12)



# tests.test_model_selection.test_learning_curve
# Tests for the LearningCurve visualizer
#
# Author:   Jason Keung
# Created:  Tues May 23 11:45:00 2017 -0400
#
# Copyright (C) 2017 The scikit-yb developers
# For license information, see LICENSE.txt
#
# ID: test_learning_curve.py jason.s.keung@gmail.com $

"""
Tests for the LearningCurve visualizer
"""

##########################################################################
# Imports
##########################################################################

import sys
import pytest
import numpy as np

from unittest.mock import patch
from tests.base import VisualTestCase

from sklearn.svm import LinearSVC
from sklearn.linear_model import Ridge
from sklearn.naive_bayes import GaussianNB, MultinomialNB
from sklearn.cluster import MiniBatchKMeans
from sklearn.preprocessing import OneHotEncoder, LabelEncoder
from sklearn.impute import SimpleImputer
from sklearn.model_selection import ShuffleSplit
from sklearn.model_selection import StratifiedKFold
from sklearn.ensemble import RandomForestClassifier
from sklearn.pipeline import Pipeline

from yellowbrick.datasets import load_mushroom, load_game
from yellowbrick.model_selection.learning_curve import *

try:
    import pandas as pd
except ImportError:
    pd = None


##########################################################################
# LearningCurve Test Cases
##########################################################################


@pytest.mark.usefixtures("classification", "regression", "clusters")
class TestLearningCurve(VisualTestCase):
    """
    Test the LearningCurve visualizer
    """

    @patch.object(LearningCurve, "draw")
    def test_fit(self, mock_draw):
        """
        Assert that fit returns self and creates expected properties
        """
        X, y = self.classification
        params = (
            "train_sizes_",
            "train_scores_",
            "train_scores_mean_",
            "train_scores_std_",
            "test_scores_",
            "test_scores_mean_",
            "test_scores_std_",
        )

        oz = LearningCurve(GaussianNB(), random_state=12)
        for param in params:
            assert not hasattr(oz, param)

        assert oz.fit(X, y) is oz
        mock_draw.assert_called_once()

        for param in params:
            assert hasattr(oz, param)

    @pytest.mark.xfail(sys.platform == "win32", reason="images not close on windows")
    def test_classifier(self):
        """
        Test image closeness on a classification dataset
        """
        X, y = self.classification

        oz = LearningCurve(
            RandomForestClassifier(random_state=21), random_state=12
        ).fit(X, y)
        oz.finalize()

        self.assert_images_similar(
            oz, tol=2.4
        )  # Increased for cross-platform compatibility (RMS 1.970)

    @pytest.mark.xfail(sys.platform == "win32", reason="images not close on windows")
    def test_regressor(self):
        """
        Test image closeness on a regression dataset
        """
        X, y = self.regression

        oz = LearningCurve(Ridge(), random_state=18)
        oz.fit(X, y)
        oz.finalize()

        self.assert_images_similar(oz)

    def test_clusters(self):
        """
        Test image closeness on a clustering dataset
        """
        X, y = self.clusters

        oz = LearningCurve(MiniBatchKMeans(random_state=281), random_state=182).fit(X)
        oz.finalize()

        self.assert_images_similar(oz, tol=10)

    @pytest.mark.xfail(sys.platform == "win32", reason="images not close on windows")
    def test_quick_method(self):
        """
        Test the learning curve quick method acts as expected
        """
        X, y = self.classification
        train_sizes = np.linspace(0.1, 1.0, 8)
        viz = learning_curve(
            GaussianNB(),
            X,
            y,
            train_sizes=train_sizes,
            cv=ShuffleSplit(n_splits=10, test_size=0.2, random_state=34),
            scoring="f1_macro",
            random_state=43,
            show=False,
        )

        self.assert_images_similar(viz)

    @pytest.mark.xfail(sys.platform == "win32", reason="images not close on windows")
    @pytest.mark.skipif(pd is None, reason="test requires pandas")
    def test_pandas_integration(self):
        """
        Test on a real dataset with pandas DataFrame and Series
        """
        data = load_mushroom(return_dataset=True)
        X, y = data.to_pandas()

        X = pd.get_dummies(X)

        assert isinstance(X, pd.DataFrame)
        assert isinstance(y, pd.Series)

        cv = StratifiedKFold(n_splits=4, shuffle=True, random_state=32)
        oz = LearningCurve(GaussianNB(), cv=cv, random_state=23)
        oz.fit(X, y)
        oz.finalize()

        self.assert_images_similar(oz)

    def test_numpy_integration(self):
        """
        Test on a real dataset with NumPy arrays
        """
        data = load_mushroom(return_dataset=True)
        X, y = data.to_numpy()

        X = OneHotEncoder().fit_transform(X).toarray()

        cv = StratifiedKFold(n_splits=4, shuffle=True, random_state=32)
        oz = LearningCurve(GaussianNB(), cv=cv, random_state=23)
        oz.fit(X, y)
        oz.finalize()

        self.assert_images_similar(oz)

    @patch.object(LearningCurve, "draw")
    def test_reshape_scores(self, mock_draw):
        """
        Test supplying an alternate CV methodology and train_sizes
        """
        X, y = self.classification

        cv = ShuffleSplit(n_splits=12, test_size=0.2, random_state=14)
        oz = LearningCurve(LinearSVC(), cv=cv, train_sizes=[0.5, 0.75, 1.0])
        oz.fit(X, y)

        assert oz.train_scores_.shape == (3, 12)
        assert oz.test_scores_.shape == (3, 12)

    def test_bad_train_sizes(self):
        """
        Test learning curve with bad input for training size.
        """
        with pytest.raises(YellowbrickValueError):
            LearningCurve(LinearSVC(), train_sizes=10000)

    def test_within_pipeline(self):
        """
        Test that visualizer can be accessed within a sklearn pipeline
        """
        X, y = load_game()

        # Encode the categorical data
        X = OneHotEncoder().fit_transform(X)
        y = LabelEncoder().fit_transform(y)

        # Create the learning curve visualizer
        cv = StratifiedKFold(n_splits=12)
        sizes = np.linspace(0.3, 1.0, 10)

        model = Pipeline(
            [
                ("imputer", SimpleImputer(missing_values=np.nan, strategy="mean")),
                (
                    "lc",
                    LearningCurve(
                        MultinomialNB(),
                        cv=cv,
                        scoring="f1_weighted",
                        train_sizes=sizes,
                        n_jobs=4,
                        random_state=42,
                    ),
                ),
            ]
        )

        model.fit(X, y)
        model["lc"].finalize()
        self.assert_images_similar(model["lc"], tol=2.0)

    def test_within_pipeline_quickmethod(self):
        """
        Test that visualizer quickmethod can be accessed within a
        sklearn pipeline
        """
        X, y = load_game()

        # Encode the categorical data
        X = OneHotEncoder().fit_transform(X)
        y = LabelEncoder().fit_transform(y)

        # Create the learning curve visualizer
        cv = StratifiedKFold(n_splits=12)
        sizes = np.linspace(0.3, 1.0, 10)

        model = Pipeline(
            [
                ("imputer", SimpleImputer(missing_values=np.nan, strategy="mean")),
                (
                    "lc",
                    learning_curve(
                        MultinomialNB(),
                        X,
                        y,
                        cv=cv,
                        scoring="f1_weighted",
                        train_sizes=sizes,
                        n_jobs=4,
                        random_state=42,
                    ),
                ),
            ]
        )
        model["lc"].finalize()
        self.assert_images_similar(model["lc"], tol=2.0)

    def test_pipeline_as_model_input(self):
        """
        Test that visualizer can handle sklearn pipeline as model input
        """
        X, y = load_game()

        # Encode the categorical data
        X = OneHotEncoder().fit_transform(X)
        y = LabelEncoder().fit_transform(y)

        # Create the learning curve visualizer
        cv = StratifiedKFold(n_splits=12)
        sizes = np.linspace(0.3, 1.0, 10)

        model = Pipeline(
            [
                ("imputer", SimpleImputer(missing_values=np.nan, strategy="mean")),
                ("nb", MultinomialNB()),
            ]
        )

        oz = LearningCurve(
            model,
            cv=cv,
            scoring="f1_weighted",
            train_sizes=sizes,
            n_jobs=4,
            random_state=42,
        )
        oz.fit(X, y)
        oz.finalize()
        self.assert_images_similar(oz, tol=2.0)

    def test_pipeline_as_model_input_quickmethod(self):
        """
        Test that visualizer can handle sklearn pipeline as model input
        within a quickmethod
        """
        X, y = load_game()

        # Encode the categorical data
        X = OneHotEncoder().fit_transform(X)
        y = LabelEncoder().fit_transform(y)

        # Create the learning curve visualizer
        cv = StratifiedKFold(n_splits=12)
        sizes = np.linspace(0.3, 1.0, 10)

        model = Pipeline(
            [
                ("imputer", SimpleImputer(missing_values=np.nan, strategy="mean")),
                ("nb", MultinomialNB()),
            ]
        )

        oz = learning_curve(
            model,
            X,
            y,
            cv=cv,
            scoring="f1_weighted",
            train_sizes=sizes,
            n_jobs=4,
            random_state=42,
        )
        self.assert_images_similar(oz, tol=2.0)



# -*- coding: utf-8 -*-
# tests.test_features.test_pca
# Tests for the PCA based feature visualizer.
#
# Author:   Carlo Morales
# Author:   Raúl Peralta Lozada
# Author:   Benjamin Bengfort
# Created:  Tue May 23 18:34:27 2017 -0400
#
# Copyright (C) 2017 The scikit-yb developers.
# For license information, see LICENSE.txt
#
# ID: test_pca.py [] cmorales@pacificmetrics.com $

"""
Tests for the PCA based feature visualizer.
"""

##########################################################################
## Imports
##########################################################################

import sys
import pytest
import numpy as np
import numpy.testing as npt

from unittest import mock
from tests.base import VisualTestCase, IS_WINDOWS_OR_CONDA

from yellowbrick.features.pca import *
from yellowbrick.exceptions import YellowbrickError, NotFitted

# Note: this can be removed when we deprecate mpl in #826
try:
    # Only available in Matplotlib >= 2.0.2
    from mpl_toolkits.axes_grid1 import make_axes_locatable
except ImportError:
    make_axes_locatable = None


##########################################################################
# PCA Tests
##########################################################################


@pytest.mark.usefixtures("discrete", "continuous")
class TestPCA(VisualTestCase):
    """
    Test the PCA visualizer
    """

    def test_single(self):
        """
        Test single target.
        """
        visualizer = PCA(random_state=1998)
        visualizer.fit(self.continuous.X)
        visualizer.transform(self.continuous.X)
        assert not hasattr(visualizer, "classes_")
        assert not hasattr(visualizer, "range_")
        self.assert_images_similar(visualizer)

    @pytest.mark.xfail(IS_WINDOWS_OR_CONDA, reason="RMS of 10.205 on miniconda")
    def test_continuous(self):
        """
        Test continuous target
        """
        visualizer = PCA(colormap="YlOrRd", random_state=2019)
        assert not hasattr(visualizer, "range_")
        visualizer.fit(*self.continuous)
        visualizer.transform(*self.continuous)
        assert hasattr(visualizer, "range_")
        assert not hasattr(visualizer, "classes_")
        visualizer.finalize()

        visualizer.cax.set_yticklabels([])

        # AppVeyor tests fail with RMS 10.085
        self.assert_images_similar(visualizer, windows_tol=10.5)

    def test_discrete(self):
        """
        Test discrete target.
        """
        classes = ["a", "b", "c", "d", "e"]
        colors = ["r", "b", "g", "m", "c"]

        visualizer = PCA(colors=colors, classes=classes, random_state=83)
        assert not hasattr(visualizer, "classes_")
        visualizer.fit(*self.discrete)
        assert hasattr(visualizer, "classes_")
        assert not hasattr(visualizer, "range_")
        visualizer.transform(*self.discrete)

        # Make sure that classes are set correctly.
        npt.assert_array_equal(visualizer.classes_, classes)

        self.assert_images_similar(visualizer)

    def test_fit(self):
        """
        Test that fit returns self.
        """
        pca = PCA()
        assert pca.fit(*self.discrete) is pca

    @pytest.mark.parametrize("n_components", [2, 3])
    def test_transform(self, n_components):
        Xprime = PCA(projection=n_components).fit_transform(*self.continuous)
        assert Xprime.shape == (500, n_components)

    def test_transform_without_fit(self):
        """
        Test that appropriate error is raised when transform called without fit.
        """
        oz = PCA(projection=3)
        msg = "instance is not fitted yet, please call fit"
        with pytest.raises(NotFitted, match=msg):
            oz.transform(*self.continuous)

    @pytest.mark.xfail(IS_WINDOWS_OR_CONDA, reason="RMS of 12.115 on miniconda")
    def test_pca_decomposition_quick_method(self):
        """
        Test the quick method PCA visualizer
        """
        visualizer = pca_decomposition(
            *self.discrete, projection=2, scale=True, random_state=28, show=False
        )

        # AppVeyor tests fail with RMS 12.115
        self.assert_images_similar(visualizer, windows_tol=12.5)

    def test_scale_true_2d(self):
        """
        Test the PCA visualizer 2 dimensions scaled.
        """
        params = {"scale": True, "projection": 2, "random_state": 9932}
        visualizer = PCA(**params).fit(*self.discrete)
        pca_array = visualizer.transform(*self.discrete)

        # Image comparison tests
        self.assert_images_similar(visualizer)

        # Assert PCA transformation occurred successfully
        assert pca_array.shape == (self.discrete.X.shape[0], 2)

    @pytest.mark.xfail(IS_WINDOWS_OR_CONDA, reason="RMS of 8.828 on miniconda")
    def test_scale_false_2d(self):
        """
        Test the PCA visualizer 2 dimensions non-scaled.
        """
        params = {"scale": False, "projection": 2, "random_state": 1229}
        visualizer = PCA(**params).fit(*self.continuous)
        pca_array = visualizer.transform(*self.continuous)
        visualizer.finalize()
        visualizer.cax.set_yticklabels([])
        # Image comparison tests
        # AppVeyor tests fail with RMS 8.180
        self.assert_images_similar(visualizer, tol=0.03, windows_tol=8.5)

        # Assert PCA transformation occurred successfully
        assert pca_array.shape == (self.continuous.X.shape[0], 2)

    def test_biplot_2d(self):
        """
        Test the PCA 2D biplot (proj_features).
        """
        params = {
            "features": list("ABCDEFGHIKLM"),
            "random_state": 67,
            "proj_features": True,
            "projection": 2,
        }
        visualizer = PCA(**params).fit(self.discrete.X)
        pca_array = visualizer.transform(self.discrete.X)

        # Image comparison tests
        self.assert_images_similar(visualizer, tol=5)

        # Assert PCA transformation occurred successfully
        assert pca_array.shape == (self.discrete.X.shape[0], 2)

    def test_scale_true_3d(self):
        """
        Test the PCA visualizer 3 dimensions scaled.
        """
        params = {"scale": True, "projection": 3, "random_state": 7382}
        visualizer = PCA(**params).fit(self.discrete.X)
        pca_array = visualizer.transform(self.discrete.X)

        # Image comparison tests
        self.assert_images_similar(visualizer)

        # Assert PCA transformation occurred successfully
        assert pca_array.shape == (self.discrete.X.shape[0], 3)

    def test_scale_false_3d(self):
        """
        Test the PCA visualizer 3 dimensions non-scaled.
        """
        params = {"scale": False, "projection": 3, "random_state": 98}
        visualizer = PCA(**params).fit(self.discrete.X)
        pca_array = visualizer.transform(self.discrete.X)

        # Image comparison tests
        self.assert_images_similar(visualizer)

        # Assert PCA transformation occurred successfully
        assert pca_array.shape == (self.discrete.X.shape[0], 3)

    @pytest.mark.xfail(
        sys.platform == "win32", reason="images not close on windows (RMSE=3)"
    )
    def test_biplot_3d(self):
        """
        Test the PCA 3D biplot (proj_features).
        """
        params = {
            "features": list("ABCDEFGHIKLM"),
            "random_state": 800,
            "proj_features": True,
            "projection": 3,
        }
        visualizer = PCA(**params).fit(*self.discrete)
        pca_array = visualizer.transform(*self.discrete)

        # Image comparison tests
        self.assert_images_similar(visualizer, tol=5)

        # Assert PCA transformation occurred successfully
        assert pca_array.shape == (self.discrete.X.shape[0], 3)

    def test_scale_true_4d_exception(self):
        """
        Test PCA visualizer 4 dimensions scaled (catch YellowbrickError).
        """
        params = {"scale": True, "projection": 4}
        msg = "Projection dimensions must be either 2 or 3"
        with pytest.raises(YellowbrickError, match=msg):
            PCA(**params)

    def test_scale_true_3d_exception(self):
        """
        Test PCA visualizer 3 dims scaled on 2 dim data set (catch ValueError).
        """
        X = np.random.normal(loc=2, size=(100, 2))
        params = {"scale": True, "projection": 3}

        e = r"n_components=3 must be between 0 and min\(n_samples, n_features\)=2"
        with pytest.raises(ValueError, match=e):
            pca = PCA(**params)
            pca.fit(X)

    @mock.patch("yellowbrick.features.pca.plt.sca", autospec=True)
    def test_alpha_param(self, mock_sca):
        """
        Test that the user can supply an alpha param on instantiation
        """
        # Instantiate a prediction error plot, provide custom alpha
        params = {"alpha": 0.3, "projection": 2, "random_state": 9932}
        visualizer = PCA(**params).fit(self.discrete.X)
        pca_array = visualizer.transform(self.discrete.X)
        assert visualizer.alpha == 0.3

        visualizer.ax = mock.MagicMock()
        visualizer.fit(self.discrete.X)
        visualizer.transform(self.discrete.X)

        # Test that alpha was passed to internal matplotlib scatterplot
        _, scatter_kwargs = visualizer.ax.scatter.call_args
        assert "alpha" in scatter_kwargs
        assert scatter_kwargs["alpha"] == 0.3
        assert pca_array.shape == (self.discrete.X.shape[0], 2)

    @pytest.mark.xfail(IS_WINDOWS_OR_CONDA, reason="RMS of 7.332 on miniconda")
    def test_colorbar(self):
        """
        Test the PCA visualizer's colorbar features.
        """
        params = {
            "scale": True,
            "projection": 2,
            "random_state": 7382,
            "color": self.discrete.y,
            "colorbar": True,
        }
        visualizer = PCA(**params).fit(*self.continuous)
        visualizer.transform(self.continuous.X, self.continuous.y)
        visualizer.finalize()
        visualizer.cax.set_yticklabels([])

        # Image comparison tests
        # AppVeyor tests fail with RMS of 7.280
        self.assert_images_similar(visualizer, windows_tol=7.5)

    @pytest.mark.xfail(IS_WINDOWS_OR_CONDA, reason="RMS of 14.515 on miniconda")
    def test_heatmap(self):
        """
        Test the PCA visualizer's heatmap features.
        """
        params = {
            "scale": True,
            "projection": 2,
            "random_state": 7382,
            "color": self.discrete.y,
            "heatmap": True,
        }
        visualizer = PCA(**params).fit(self.discrete.X, self.discrete.y)
        visualizer.transform(self.discrete.X, self.discrete.y)
        visualizer.finalize()
        # TODO: manually modifying ticks should be removed after #916 is fixed
        visualizer.lax.set_xticks([])
        visualizer.lax.set_yticks([])
        visualizer.lax.set_xticks([], minor=True)
        visualizer.uax.set_xticklabels([])

        # Image comparison tests
        # AppVeyor tests fail with RMS 14.492
        self.assert_images_similar(visualizer, windows_tol=14.5)

    @pytest.mark.xfail(IS_WINDOWS_OR_CONDA, reason="RMS of 10.987 on miniconda")
    def test_colorbar_heatmap(self):
        """
        Test the PCA visualizer with both colorbar and heatmap.
        """
        params = {
            "scale": True,
            "projection": 2,
            "random_state": 7382,
            "color": self.discrete.y,
            "colorbar": True,
            "heatmap": True,
        }
        visualizer = PCA(**params).fit(self.continuous.X, self.continuous.y)
        visualizer.transform(self.continuous.X, self.continuous.y)
        visualizer.finalize()
        # TODO: manually modifying ticks should be removed after #916 is fixed
        visualizer.lax.set_xticks([])
        visualizer.lax.set_yticks([])
        visualizer.lax.set_xticks([], minor=True)
        visualizer.uax.set_xticklabels([])
        visualizer.cax.set_yticklabels([])

        # Image comparison tests
        # AppVeyor tests fail with RMS 10.331
        self.assert_images_similar(visualizer, windows_tol=10.5)

    def test_3d_heatmap_enabled_error(self):
        """
        Assert an exception if colorbar and heatmap is enabled with 3-dimensions
        """
        with pytest.raises(YellowbrickValueError):
            PCA(projection=3, heatmap=True)

    @pytest.mark.skipif(
        make_axes_locatable is not None, reason="requires matplotlib <= 2.0.1"
    )
    def test_matplotlib_version_error():
        """
        Assert an exception is raised with incompatible matplotlib versions
        """
        with pytest.raises(YellowbrickValueError):
            PCA(colorbar=True, heatmap=True)



# tests.test_model_selection.test_rfecv
# Tests for the RFECV visualizer
#
# Author:  Benjamin Bengfort
# Created: Tue Apr 03 17:35:16 2018 -0400
#
# Copyright (C) 2018 The scikit-yb developers
# For license information, see LICENSE.txt
#
# ID: test_rfecv.py [a4599db] rebeccabilbro@users.noreply.github.com $

"""
Tests for the RFECV visualizer
"""

##########################################################################
## Imports
##########################################################################

import sys
import pytest
import numpy as np
import numpy.testing as npt

from unittest.mock import patch

from tests.fixtures import Dataset
from tests.base import VisualTestCase
from yellowbrick.model_selection.rfecv import *
from yellowbrick.datasets import load_occupancy
from yellowbrick.exceptions import YellowbrickValueError

from sklearn.svm import SVC
from sklearn.datasets import make_classification
from sklearn.model_selection import ShuffleSplit
from sklearn.model_selection import StratifiedKFold
from sklearn.linear_model import LogisticRegression
from sklearn.ensemble import RandomForestClassifier

try:
    import pandas as pd
except ImportError:
    pd = None


##########################################################################
## Fixtures
##########################################################################


@pytest.fixture(scope="class")
def dataset(request):
    """
    Creates a multiclass classification dataset fixture for RFECV
    """
    X, y = make_classification(
        n_samples=300,
        n_features=5,
        n_informative=3,
        n_repeated=0,
        n_classes=4,
        n_clusters_per_class=1,
        random_state=0,
    )

    dataset = Dataset(X, y)
    request.cls.dataset = dataset


##########################################################################
## Test Cases
##########################################################################


@pytest.mark.usefixtures("dataset")
class TestRFECV(VisualTestCase):
    """
    Test the RFECV visualizer
    """

    @patch.object(RFECV, "draw")
    def test_fit(self, mock_draw):
        """
        Assert that fit returns self and creates expected properties with NB
        """
        X, y = self.dataset
        params = (
            "n_features_",
            "support_",
            "ranking_",
            "cv_scores_",
            "rfe_estimator_",
            "n_feature_subsets_",
        )

        rf = RandomForestClassifier()
        oz = RFECV(rf)
        for param in params:
            assert not hasattr(oz, param)

        # Assert original estimator is wrapped
        assert oz._wrapped is rf

        assert oz.fit(X, y) is oz
        mock_draw.assert_called_once()

        for param in params:
            assert hasattr(oz, param)

        # Assert rfe estimator is now wrapped
        assert oz._wrapped is not rf
        assert oz._wrapped is oz.rfe_estimator_

    @pytest.mark.xfail(sys.platform == "win32", reason="images not close on windows")
    def test_rfecv_classification(self):
        """
        Test image closeness on a classification dataset with an SVM
        """
        cv = ShuffleSplit(3, random_state=21)
        oz = RFECV(SVC(kernel="linear", C=1), cv=cv)
        oz.fit(self.dataset.X, self.dataset.y)
        oz.finalize()

        self.assert_images_similar(oz, remove_legend=True)

    @pytest.mark.xfail(sys.platform == "win32", reason="images not close on windows")
    @pytest.mark.filterwarnings("ignore:F-score is ill-defined")
    def test_quick_method(self):
        """
        Test the rfecv quick method works with LogisticRegression
        """
        cv = ShuffleSplit(2, random_state=14)
        model = LogisticRegression()
        X, y = self.dataset

        viz = rfecv(model, X, y, step=2, cv=cv, scoring="f1_weighted", show=False)

        self.assert_images_similar(viz, remove_legend=True)

    @pytest.mark.xfail(sys.platform == "win32", reason="images not close on windows")
    @pytest.mark.skipif(pd is None, reason="test requires pandas")
    def test_pandas_integration(self):
        """
        Test on a real dataset with pandas DataFrame and Series
        """
        data = load_occupancy(return_dataset=True)
        X, y = data.to_pandas()

        # Use only the first 100 samples so the test will run faster
        X_t = X[:100]
        y_t = y[:100]

        assert isinstance(X_t, pd.DataFrame)
        assert isinstance(y_t, pd.Series)

        cv = StratifiedKFold(n_splits=4, shuffle=True, random_state=32)
        oz = RFECV(RandomForestClassifier(random_state=83), cv=cv)
        oz.fit(X_t, y_t)
        oz.finalize()

        self.assert_images_similar(oz, remove_legend=True)

    @pytest.mark.xfail(sys.platform == "win32", reason="images not close on windows")
    def test_numpy_integration(self):
        """
        Test on a real dataset with numpy ndarray
        """
        data = load_occupancy(return_dataset=True)
        X, y = data.to_numpy()

        # Use only the first 100 samples so the test will run faster
        X_t = X[:100]
        y_t = y[:100]

        assert isinstance(X_t, np.ndarray)
        assert isinstance(y_t, np.ndarray)

        cv = StratifiedKFold(n_splits=4, shuffle=True, random_state=32)
        oz = RFECV(RandomForestClassifier(random_state=83), cv=cv)
        oz.fit(X_t, y_t)
        oz.finalize()

        self.assert_images_similar(oz, remove_legend=True)

    @pytest.mark.parametrize("step", [0, -1, -5])
    def test_invalid_step(self, step):
        """
        Test step hyperparam validation
        """
        with pytest.raises(YellowbrickValueError, match="step must be >0"):
            oz = RFECV(SVC(kernel="linear"), step=step)
            oz.fit(self.dataset.X, self.dataset.y)

    def test_rfecv_step(self):
        """
        Test RFECV step=5 with LogisticRegression
        """
        X, y = make_classification(
            n_samples=200,
            n_features=30,
            n_informative=18,
            n_redundant=6,
            n_repeated=0,
            n_classes=8,
            n_clusters_per_class=1,
            random_state=0,
        )

        oz = RFECV(LogisticRegression(random_state=32), step=5).fit(X, y)
        assert hasattr(oz, "n_feature_subsets_")
        npt.assert_array_equal(oz.n_feature_subsets_, np.arange(1, 35, 5))

        oz.finalize()
        tol = 1.75 if sys.platform == "win32" else 0.25
        self.assert_images_similar(oz, tol=tol, remove_legend=True)



# tests.test_boundaries
# Ensure that the decision boundary visualizations work.
#
# Author:   Author:   Nathan Danielsen <nathan.danielsen@gmail.com>
# Created:  Sun Mar 19 13:01:29 2017 -0400
#
# Copyright (C) 2017 The scikit-yb developers
# For license information, see LICENSE.txt
#
# ID: test_boundaries.py [a60bc41] nathan.danielsen@gmail.com $
"""
Ensure that the Decision Boundary visualizations work.
"""

##########################################################################
# Imports
##########################################################################

import sys
import pytest
import numpy as np

from unittest import mock
from tests.base import VisualTestCase

from yellowbrick.contrib.classifier import *
from yellowbrick.exceptions import YellowbrickTypeError
from yellowbrick.exceptions import YellowbrickValueError

from sklearn import datasets
from sklearn import neighbors
from sklearn import naive_bayes

try:
    import pandas as pd
except ImportError:
    pd = None


##########################################################################
# Data
##########################################################################

X = np.array(
    [
        [2.318, 2.727, 4.260, 7.212, 4.792],
        [2.315, 2.726, 4.295, 7.140, 4.783],
        [2.315, 2.724, 4.260, 7.135, 4.779],
        [2.110, 3.609, 4.330, 7.985, 5.595],
        [2.110, 3.626, 4.330, 8.203, 5.621],
        [2.110, 3.620, 4.470, 8.210, 5.612],
        [2.318, 2.727, 4.260, 7.212, 4.792],
        [2.315, 2.726, 4.295, 7.140, 4.783],
        [2.315, 2.724, 4.260, 7.135, 4.779],
        [2.110, 3.609, 4.330, 7.985, 5.595],
        [2.110, 3.626, 4.330, 8.203, 5.621],
        [2.110, 3.620, 4.470, 8.210, 5.612],
    ]
)

y = np.array([1, 2, 1, 2, 1, 0, 0, 1, 3, 1, 3, 2])

X_two_cols = X[:, :2]

##########################################################################
# Residuals Plots test case
##########################################################################


@pytest.mark.filterwarnings("ignore")
class TestDecisionBoundariesVisualizer(VisualTestCase):
    """
    Test DecisionBoundariesVisualizer
    """

    def test_decision_boundaries(self):
        """
        Assert no errors during kNN DecisionBoundariesVisualizer integration
        """
        model = neighbors.KNeighborsClassifier(3)
        viz = DecisionViz(model)
        viz.fit_draw_show(X_two_cols, y=y)

    def test_init(self):
        """
        Test correct initialization of the internal state
        """
        model = neighbors.KNeighborsClassifier(3)
        viz = DecisionBoundariesVisualizer(model)

        assert viz.step_size == 0.0025
        assert viz.name == "KNeighborsClassifier"
        assert viz.estimator is model

        assert not hasattr(viz, "classes_")
        assert viz.features_ is None
        assert viz.markers is not None
        assert viz.scatter_alpha is not None
        assert viz.show_scatter is True

        assert viz.Z is None
        assert viz.xx is None
        assert viz.yy is None
        assert viz.class_labels is None
        assert viz.title is None
        assert viz.x is None
        assert viz.y is None

    def test_scatter_xy_and_features_raise_error(self):
        """
        Assert that x,y and features will raise error
        """
        model = neighbors.KNeighborsClassifier(3)
        features = ["temperature", "relative_humidity", "light"]

        with pytest.raises(YellowbrickValueError):
            DecisionBoundariesVisualizer(model, features=features, x="one", y="two")

    def test_scatter_xy_changes_to_features(self):
        """
        Assert that x,y and features will raise error
        """
        model = neighbors.KNeighborsClassifier(3)
        visualizer = DecisionBoundariesVisualizer(model, x="one", y="two")
        assert visualizer.features_ == ["one", "two"]

    def test_fit(self):
        """
        Testing the fit method works as expected
        """
        model = neighbors.KNeighborsClassifier(3)
        model.fit = mock.MagicMock()
        model.predict = mock.MagicMock()

        viz = DecisionBoundariesVisualizer(model)
        fitted_viz = viz.fit(X_two_cols, y=y)

        # assert that classes and labels are established
        assert fitted_viz.classes_ == {0: "0", 1: "1", 2: "2", 3: "3"}
        assert fitted_viz.features_ == ["Feature One", "Feature Two"]

        # assert that the fit method is called
        model.fit.assert_called_once_with(X_two_cols, y)
        # mock object is called twice in predict and reshape
        assert len(model.predict.mock_calls) == 2

        # test that attrs are set
        assert fitted_viz.ax is not None
        assert fitted_viz.Z_shape is not None

    def test_fit_class_labels(self):
        """
        Test fit with class labels specified
        """
        model = neighbors.KNeighborsClassifier(3)
        viz = DecisionBoundariesVisualizer(
            model, classes=["one", "two", "three", "four"]
        )
        fitted_viz = viz.fit(X_two_cols, y=y)
        assert fitted_viz.classes_ == {
            "three": "2",
            "four": "3",
            "two": "1",
            "one": "0",
        }

    def test_fit_class_labels_class_names_edge_case(self):
        """
        Edge case that more class labels are defined than in datatset
        """
        model = neighbors.KNeighborsClassifier(3)
        viz = DecisionBoundariesVisualizer(
            model, classes=["one", "two", "three", "four", "five"]
        )

        with pytest.raises(YellowbrickTypeError):
            viz.fit(X_two_cols, y=y)

    def test_fit_features_assignment_None(self):
        """
        Test fit when features is None
        """
        model = neighbors.KNeighborsClassifier(3)
        viz = DecisionBoundariesVisualizer(model)
        assert viz.features_ is None
        fitted_viz = viz.fit(X_two_cols, y=y)
        assert fitted_viz.features_ == ["Feature One", "Feature Two"]

    def test_fit_features_assignment(self):
        """
        Test fit when features are specified
        """
        model = neighbors.KNeighborsClassifier(3)
        viz = DecisionBoundariesVisualizer(model, features=["one", "two"])
        fitted_viz = viz.fit(X_two_cols, y=y)
        assert fitted_viz.features_ == ["one", "two"]

    @mock.patch("yellowbrick.contrib.classifier.boundaries.OrderedDict")
    def test_draw_ordereddict_calls(self, mock_odict):
        """
        Test draw with calls to ordered dict
        """
        mock_odict.return_value = {}
        model = neighbors.KNeighborsClassifier(3)
        viz = DecisionBoundariesVisualizer(model, features=["one", "two"])

        with pytest.raises(KeyError):
            viz.fit_draw(X_two_cols, y=y)

        assert len(mock_odict.mock_calls) == 2

    @mock.patch("yellowbrick.contrib.classifier.boundaries.resolve_colors")
    def test_draw_ordereddict_calls_one(self, mock_resolve_colors):
        """
        Test ordered dict calls resolve colors
        """
        mock_resolve_colors.return_value = []
        model = neighbors.KNeighborsClassifier(3)
        viz = DecisionBoundariesVisualizer(model, features=["one", "two"])

        with pytest.raises(StopIteration):
            viz.fit_draw(X_two_cols, y=y)

        assert len(mock_resolve_colors.mock_calls) == 1

    def test_draw_ax_show_scatter_true(self):
        """
        Test that the matplotlib functions are being called
        """
        model = neighbors.KNeighborsClassifier(3)
        viz = DecisionBoundariesVisualizer(model, features=["one", "two"])
        fitted_viz = viz.fit(X_two_cols, y=y)
        fitted_viz.ax = mock.Mock()
        fitted_viz.ax.pcolormesh = mock.MagicMock()
        fitted_viz.ax.scatter = mock.MagicMock()
        fitted_viz.ax.legend = mock.MagicMock()

        fitted_viz.draw(X_two_cols, y=y)
        assert len(fitted_viz.ax.pcolormesh.mock_calls) == 1
        assert len(fitted_viz.ax.scatter.mock_calls) == 4
        assert len(fitted_viz.ax.legend.mock_calls) == 0

    def test_draw_ax_show_scatter_False(self):
        """
        Test that the matplotlib called when the scatter plot isn't drawn
        """
        model = neighbors.KNeighborsClassifier(3)
        viz = DecisionBoundariesVisualizer(
            model, features=["one", "two"], show_scatter=False
        )
        fitted_viz = viz.fit(X_two_cols, y=y)
        fitted_viz.ax = mock.Mock()
        fitted_viz.ax.pcolormesh = mock.MagicMock()
        fitted_viz.ax.scatter = mock.MagicMock()
        fitted_viz.ax.legend = mock.MagicMock()

        fitted_viz.draw(X_two_cols, y=y)
        assert len(fitted_viz.ax.pcolormesh.mock_calls) == 1
        assert len(fitted_viz.ax.scatter.mock_calls) == 0
        assert len(fitted_viz.ax.legend.mock_calls) == 1

    def test_finalize(self):
        """
        Test the finalize method works as expected
        """
        model = neighbors.KNeighborsClassifier(3)
        viz = DecisionBoundariesVisualizer(
            model, features=["one", "two"], show_scatter=False
        )
        fitted_viz = viz.fit(X_two_cols, y=y)
        fitted_viz.draw(X_two_cols, y=y)

        fitted_viz.ax = mock.Mock()
        fitted_viz.ax.legend = mock.MagicMock()
        fitted_viz.ax.set_xlabel = mock.MagicMock()
        fitted_viz.ax.set_ylabel = mock.MagicMock()

        fitted_viz.finalize()

        fitted_viz.ax.legend.assert_called_once_with(loc="best", frameon=True)
        fitted_viz.ax.set_xlabel.assert_called_once_with("one")
        fitted_viz.ax.set_ylabel.assert_called_once_with("two")

    def test_fit_draw(self):
        """
        Test fit draw shortcut
        """
        model = neighbors.KNeighborsClassifier(3)
        viz = DecisionBoundariesVisualizer(
            model, features=["one", "two"], show_scatter=False
        )

        viz.fit = mock.Mock()
        viz.draw = mock.Mock()

        viz.fit_draw(X_two_cols, y=y)

        viz.fit.assert_called_once_with(X_two_cols, y)
        viz.draw.assert_called_once_with(X_two_cols, y)

    def test_fit_draw_show(self):
        """
        Test fit draw show shortcut
        """
        model = neighbors.KNeighborsClassifier(3)
        viz = DecisionBoundariesVisualizer(
            model, features=["one", "two"], show_scatter=False
        )

        viz.fit = mock.Mock()
        viz.draw = mock.Mock()
        viz.show = mock.Mock()

        viz.fit_draw_show(X_two_cols, y=y)

        viz.fit.assert_called_once_with(X_two_cols, y)
        viz.draw.assert_called_once_with(X_two_cols, y)
        viz.show.assert_called_once_with()

    @pytest.mark.xfail(reason="numpy structured arrays have changed since v1.14")
    def test_integrated_plot_numpy_named_arrays(self):
        """
        Test integration of visualizer with numpy named arrays
        """
        model = naive_bayes.MultinomialNB()

        X = np.array(
            [
                (1.1, 9.52, 1.23, 0.86, 7.89, 0.13),
                (3.4, 2.84, 8.65, 0.45, 7.43, 0.16),
                (1.2, 3.22, 6.56, 0.24, 3.45, 0.17),
                (3.8, 6.18, 2.45, 0.28, 2.53, 0.13),
                (5.1, 9.12, 1.06, 0.19, 1.43, 0.13),
                (4.4, 8.84, 4.97, 0.98, 1.35, 0.13),
                (3.2, 3.22, 5.03, 0.68, 3.53, 0.32),
                (7.8, 2.18, 6.87, 0.35, 3.25, 0.38),
            ],
            dtype=[
                ("a", "<f8"),
                ("b", "<f8"),
                ("c", "<f8"),
                ("d", "<f8"),
                ("e", "<f8"),
                ("f", "<f8"),
            ],
        )

        y = np.array([1, 1, 0, 1, 0, 0, 1, 0])

        visualizer = DecisionBoundariesVisualizer(model, features=["a", "f"])
        visualizer.fit_draw_show(X, y=y)
        self.assertEquals(visualizer.features_, ["a", "f"])
        self.assert_images_similar(visualizer)

    def test_integrated_scatter_numpy_arrays_no_names(self):
        """
        Test integration of visualizer with numpy arrays
        """
        model = neighbors.KNeighborsClassifier(n_neighbors=3)

        visualizer = DecisionBoundariesVisualizer(model, features=[1, 2])
        visualizer.fit_draw_show(X, y)
        assert visualizer.features_ == [1, 2]

    @pytest.mark.xfail(sys.platform == "win32", reason="images not close on windows")
    @pytest.mark.skipif(pd is None, reason="test requires pandas")
    def test_real_data_set_viz(self):
        """
        Test integration of visualizer with pandas dataset
        """
        model = naive_bayes.MultinomialNB()

        data = datasets.load_iris()
        feature_names = [name.replace(" ", "_") for name in data.feature_names]
        df = pd.DataFrame(data.data, columns=feature_names)
        X = df[["sepal_length_(cm)", "sepal_width_(cm)"]].values
        y = data.target

        visualizer = DecisionBoundariesVisualizer(model)
        visualizer.fit_draw_show(X, y)
        self.assert_images_similar(visualizer)

    @pytest.mark.skipif(pd is None, reason="test requires pandas")
    def test_quick_method(self):
        """
        Test quick function shortcut of visualizer
        """
        model = naive_bayes.MultinomialNB()

        data = datasets.load_iris()
        feature_names = [name.replace(" ", "_") for name in data.feature_names]
        df = pd.DataFrame(data.data, columns=feature_names)
        X = df[["sepal_length_(cm)", "sepal_width_(cm)"]].values
        y = data.target

        decisionviz(model, X, y)



# tests.test_text.test_base
# Tests for the text visualization base classes
#
# Author:   Benjamin Bengfort
# Created:  Mon Feb 20 06:34:50 2017 -0500
#
# Copyright (C) 2016 The scikit-yb developers
# For license information, see LICENSE.txt
#
# ID: test_base.py [6aa9198] benjamin@bengfort.com $

"""
Tests for the text visualization base classes
"""

##########################################################################
## Imports
##########################################################################

from yellowbrick.base import *
from yellowbrick.text.base import *
from sklearn.base import BaseEstimator, TransformerMixin


##########################################################################
## TextVisualizer Base Tests
##########################################################################


class TestTextVisualizerBase(object):
    def test_subclass(self):
        """
        Assert the text visualizer is subclassed correctly
        """
        visualizer = TextVisualizer()
        assert isinstance(visualizer, TransformerMixin)
        assert isinstance(visualizer, BaseEstimator)
        assert isinstance(visualizer, Visualizer)



# tests.test_classifier.test_rocauc
# Testing for the ROCAUC visualizer
#
# Author:   Benjamin Bengfort
# Author:   Rebecca Bilbro
# Created:  Tue May 23 13:41:55 2017 -0700
#
# Copyright (C) 2017 The scikit-yb developers
# For license information, see LICENSE.txt
#
# ID: test_rocauc.py [] benjamin@bengfort.com $

"""
Testing for the ROCAUC visualizer
"""

##########################################################################
## Imports
##########################################################################

import pytest
import numpy as np
import numpy.testing as npt

from unittest.mock import patch
from tests.base import VisualTestCase

from yellowbrick.classifier.rocauc import *
from yellowbrick.exceptions import ModelError
from yellowbrick.datasets import load_occupancy

from sklearn.svm import LinearSVC, SVC
from sklearn.naive_bayes import GaussianNB
from sklearn.linear_model import LogisticRegression
from sklearn.base import BaseEstimator, ClassifierMixin
from sklearn.model_selection import train_test_split as tts
from sklearn.ensemble import RandomForestClassifier, AdaBoostClassifier
from sklearn.preprocessing import MinMaxScaler
from sklearn.pipeline import Pipeline

try:
    import pandas as pd
except ImportError:
    pd = None


##########################################################################
## Fixtures
##########################################################################


class FakeClassifier(BaseEstimator, ClassifierMixin):
    """
    A fake classifier for testing noops on the visualizer.
    """

    pass


def assert_valid_rocauc_scores(visualizer, nscores=4):
    """
    Assertion helper to ensure scores are correctly computed
    """
    __tracebackhide__ = True
    assert len(visualizer.fpr.keys()) == nscores
    assert len(visualizer.tpr.keys()) == nscores
    assert len(visualizer.roc_auc.keys()) == nscores

    for k in (0, 1, "micro", "macro"):
        assert k in visualizer.fpr
        assert k in visualizer.tpr
        assert k in visualizer.roc_auc
        assert len(visualizer.fpr[k]) == len(visualizer.tpr[k])
        assert 0.0 < visualizer.roc_auc[k] < 1.0


##########################################################################
##  Tests
##########################################################################


@pytest.mark.usefixtures("binary", "multiclass")
class TestROCAUC(VisualTestCase):
    """
    Test ROCAUC visualizer
    """

    def test_binary_probability(self):
        """
        Test ROCAUC with a binary classifier with a predict_proba function
        """
        # Create and fit the visualizer
        visualizer = ROCAUC(RandomForestClassifier(random_state=42))
        visualizer.fit(self.binary.X.train, self.binary.y.train)

        # Score the visualizer
        s = visualizer.score(self.binary.X.test, self.binary.y.test)

        # Test that score method successfully returns a value between 0 and 1
        assert 0 <= s <= 1

        # Check the scores
        assert_valid_rocauc_scores(visualizer)

        # Compare the images
        visualizer.finalize()
        self.assert_images_similar(visualizer, tol=6.9, windows_tol=10)

    def test_binary_probability_decision(self):
        """
        Test ROCAUC with a binary classifier with both decision & predict_proba
        """
        # Create and fit the visualizer
        visualizer = ROCAUC(AdaBoostClassifier())
        visualizer.fit(self.binary.X.train, self.binary.y.train)

        # Score the visualizer
        s = visualizer.score(self.binary.X.test, self.binary.y.test)

        # Test that score method successfully returns a value between 0 and 1
        assert 0 <= s <= 1

        # Check the scores
        assert_valid_rocauc_scores(visualizer)

        # Compare the images
        visualizer.finalize()
        self.assert_images_similar(visualizer, tol=0.1, windows_tol=10)

    def test_binary_probability_decision_single_curve(self):
        """
        Test ROCAUC binary classifier with decision & predict_proba, per_class=False
        """
        # Create and fit the visualizer
        visualizer = ROCAUC(
            AdaBoostClassifier(), micro=False, macro=False, per_class=False
        )
        visualizer.fit(self.binary.X.train, self.binary.y.train)

        # Score the visualizer
        s = visualizer.score(self.binary.X.test, self.binary.y.test)

        # Test that score method successfully returns a value between 0 and 1
        assert 0 <= s <= 1

        # Check the scores
        assert len(visualizer.fpr.keys()) == 1
        assert len(visualizer.tpr.keys()) == 1
        assert len(visualizer.roc_auc.keys()) == 1

        # Compare the images
        visualizer.finalize()
        self.assert_images_similar(visualizer, tol=0.1, windows_tol=10)

    def test_binary_decision(self):
        """
        Test ROCAUC with a binary classifier with a decision_function
        """
        # Create and fit the visualizer
        visualizer = ROCAUC(
            LinearSVC(random_state=42), micro=False, macro=False, per_class=False
        )
        visualizer.fit(self.binary.X.train, self.binary.y.train)

        # Score the visualizer
        s = visualizer.score(self.binary.X.test, self.binary.y.test)

        # Test that score method successfully returns a value between 0 and 1
        assert 0 <= s <= 1

        # Check the scores
        assert len(visualizer.fpr.keys()) == 1
        assert len(visualizer.tpr.keys()) == 1
        assert len(visualizer.roc_auc.keys()) == 1

        # Compare the images
        # NOTE: increased tolerance for both AppVeyor and Travis CI tests
        visualizer.finalize()
        self.assert_images_similar(visualizer, tol=10)

    def test_binary_decision_per_class(self):
        """
        Test ROCAUC with a binary classifier with a decision_function
        """
        # Create and fit the visualizer
        visualizer = ROCAUC(
            LinearSVC(random_state=42), micro=False, macro=False, per_class=True
        )
        visualizer.fit(self.binary.X.train, self.binary.y.train)

        # Score the visualizer
        s = visualizer.score(self.binary.X.test, self.binary.y.test)

        # Test that score method successfully returns a value between 0 and 1
        assert 0 <= s <= 1

        # Check the scores
        assert len(visualizer.fpr.keys()) == 2
        assert len(visualizer.tpr.keys()) == 2
        assert len(visualizer.roc_auc.keys()) == 2

        # Compare the images
        # NOTE: increased tolerance for both AppVeyor and Travis CI tests
        visualizer.finalize()
        self.assert_images_similar(visualizer, tol=10)

    def test_binary_micro_error(self):
        """
        Test ROCAUC to see if _binary_decision with micro = True raises an error
        """
        # Create visualizer with a linear model to force a binary decision
        visualizer = ROCAUC(LinearSVC(random_state=42), micro=True, per_class=False)
        visualizer.fit(self.binary.X.train, self.binary.y.train)

        # Ensure score raises error (micro curves aren't defined for binary decisions)
        with pytest.raises(ModelError):
            visualizer.score(self.binary.X.test, self.binary.y.test)

    def test_binary_macro_error(self):
        """
        Test ROCAUC to see if _binary_decision with macro = True raises an error
        """
        # Create visualizer with a linear model to force a binary decision
        visualizer = ROCAUC(LinearSVC(random_state=42), macro=True, per_class=False)
        visualizer.fit(self.binary.X.train, self.binary.y.train)

        # Ensure score raises error (macro curves aren't defined for binary decisions)
        with pytest.raises(ModelError):
            visualizer.score(self.binary.X.test, self.binary.y.test)

    def test_multiclass_rocauc(self):
        """
        Test ROCAUC with a multiclass classifier
        """
        # Create and fit the visualizer
        visualizer = ROCAUC(GaussianNB())
        visualizer.fit(self.multiclass.X.train, self.multiclass.y.train)

        # Score the visualizer
        s = visualizer.score(self.multiclass.X.test, self.multiclass.y.test)

        # Test that score method successfully returns a value between 0 and 1
        assert 0 <= s <= 1

        # Check the scores
        assert_valid_rocauc_scores(visualizer, nscores=8)

        # Compare the images
        visualizer.finalize()
        self.assert_images_similar(visualizer, tol=0.1, windows_tol=10)

    def test_rocauc_no_classes(self):
        """
        Test ROCAUC without per-class curves
        """
        # Create and fit the visualizer
        visualizer = ROCAUC(GaussianNB(), per_class=False)
        visualizer.fit(self.multiclass.X.train, self.multiclass.y.train)

        # Score the visualizer (should be the micro average)
        s = visualizer.score(self.multiclass.X.test, self.multiclass.y.test)
        assert s == pytest.approx(0.77303, abs=1e-4)

        # Assert that there still are per-class scores
        for c in (0, 1):
            assert c in visualizer.fpr
            assert c in visualizer.tpr
            assert c in visualizer.roc_auc

        # Compare the images
        visualizer.finalize()
        self.assert_images_similar(visualizer, tol=0.1, windows_tol=10)

    def test_rocauc_no_curves(self):
        """
        Test ROCAUC with no curves specified at all
        """
        # Create and fit the visualizer
        visualizer = ROCAUC(GaussianNB(), per_class=False, macro=False, micro=False)
        visualizer.fit(self.multiclass.X.train, self.multiclass.y.train)

        # Attempt to score the visualizer
        with pytest.raises(YellowbrickValueError, match="no curves will be drawn"):
            visualizer.score(self.multiclass.X.test, self.multiclass.y.test)

    def test_rocauc_quickmethod(self):
        """
        Test the ROCAUC quick method
        """
        X, y = load_occupancy(return_dataset=True).to_numpy()
        model = LogisticRegression()

        # compare the images
        visualizer = roc_auc(model, X, y, show=False)
        self.assert_images_similar(visualizer)

    @pytest.mark.skipif(pd is None, reason="test requires pandas")
    def test_pandas_integration(self):
        """
        Test the ROCAUC with Pandas dataframe
        """
        X, y = load_occupancy(return_dataset=True).to_pandas()

        # Create train/test splits
        splits = tts(X, y, test_size=0.2, random_state=4512)
        X_train, X_test, y_train, y_test = splits

        visualizer = ROCAUC(GaussianNB())
        visualizer.fit(X_train, y_train)
        visualizer.score(X_test, y_test)

        # Compare the images
        visualizer.finalize()
        self.assert_images_similar(visualizer)

    def test_rocauc_no_micro(self):
        """
        Test ROCAUC without a micro average
        """
        # Create and fit the visualizer
        visualizer = ROCAUC(LogisticRegression(), micro=False)
        visualizer.fit(self.binary.X.train, self.binary.y.train)

        # Score the visualizer (should be the macro average)
        s = visualizer.score(self.binary.X.test, self.binary.y.test)
        assert s == pytest.approx(0.8661, abs=1e-4)

        # Assert that there is no micro score
        assert "micro" not in visualizer.fpr
        assert "micro" not in visualizer.tpr
        assert "micro" not in visualizer.roc_auc

        # Compare the images
        visualizer.finalize()
        self.assert_images_similar(visualizer, tol=0.1, windows_tol=10)

    def test_rocauc_no_macro(self):
        """
        Test ROCAUC without a macro average
        """
        # Create and fit the visualizer
        visualizer = ROCAUC(LogisticRegression(), macro=False)
        visualizer.fit(self.binary.X.train, self.binary.y.train)

        # Score the visualizer (should be the micro average)
        s = visualizer.score(self.binary.X.test, self.binary.y.test)
        assert s == pytest.approx(0.8573, abs=1e-4)

        # Assert that there is no macro score
        assert "macro" not in visualizer.fpr
        assert "macro" not in visualizer.tpr
        assert "macro" not in visualizer.roc_auc

        # Compare the images
        visualizer.finalize()
        self.assert_images_similar(visualizer, tol=0.1, windows_tol=10)

    def test_rocauc_no_macro_no_micro(self):
        """
        Test ROCAUC without a macro or micro average
        """
        # Create and fit the visualizer
        visualizer = ROCAUC(LogisticRegression(), macro=False, micro=False)
        visualizer.fit(self.binary.X.train, self.binary.y.train)

        # Score the visualizer (should be the F1 score)
        s = visualizer.score(self.binary.X.test, self.binary.y.test)
        assert s == pytest.approx(0.8)

        # Assert that there is no macro score
        assert "macro" not in visualizer.fpr
        assert "macro" not in visualizer.tpr
        assert "macro" not in visualizer.roc_auc

        # Assert that there is no micro score
        assert "micro" not in visualizer.fpr
        assert "micro" not in visualizer.tpr
        assert "micro" not in visualizer.roc_auc

        # Compare the images
        visualizer.finalize()
        self.assert_images_similar(visualizer, tol=0.1, windows_tol=10)

    def test_rocauc_label_encoded(self):
        """
        Test ROCAUC with a target specifying a list of classes as strings
        """
        class_labels = ["a", "b", "c", "d", "e", "f"]

        # Create and fit the visualizer
        visualizer = ROCAUC(LogisticRegression(), classes=class_labels)
        visualizer.fit(self.multiclass.X.train, self.multiclass.y.train)

        # Score the visualizer
        visualizer.score(self.multiclass.X.test, self.multiclass.y.test)
        assert list(visualizer.classes_) == class_labels

    def test_rocauc_not_label_encoded(self):
        """
        Test ROCAUC with a target whose classes are unencoded strings before scoring
        """
        # Map numeric targets to strings
        classes = {0: "a", 1: "b", 2: "c", 3: "d", 4: "e", 5: "f"}
        y_train = np.array([classes[yi] for yi in self.multiclass.y.train])
        y_test = np.array([classes[yi] for yi in self.multiclass.y.test])

        # Create and fit the visualizer
        visualizer = ROCAUC(LogisticRegression())
        visualizer.fit(self.multiclass.X.train, y_train)

        # Confirm that y_train and y_test have the same targets before calling score
        assert set(y_train) == set(y_test)

    def test_binary_decision_function_rocauc(self):
        """
        Test ROCAUC with binary classifiers that have a decision function
        """
        # Load the model and assert there is no predict_proba method.
        model = LinearSVC()
        with pytest.raises(AttributeError):
            model.predict_proba

        # Fit model and visualizer
        visualizer = ROCAUC(model)
        visualizer.fit(self.binary.X.train, self.binary.y.train)

        # First 10 expected values in the y_scores
        first_ten_expected = np.asarray(
            [-0.092, 0.019, -0.751, -0.838, 0.183, -0.344, -1.019, 2.203, 1.415, -0.529]
        )

        # Get the predict_proba scores and evaluate
        y_scores = visualizer._get_y_scores(self.binary.X.train)

        # Check to see if the first 10 y_scores match the expected
        npt.assert_array_almost_equal(y_scores[:10], first_ten_expected, decimal=1)

    def test_binary_false_decision_function_error(self):
        """
        Test binary decision_function model raises error when the binary param is False
        """
        # Create and fit the visualizer
        visualizer = ROCAUC(LinearSVC(random_state=42), binary=False)
        visualizer.fit(self.binary.X.train, self.binary.y.train)

        # Ensure score raises error
        # (only binary curve defined for binary decisions with decision_function clf)
        with pytest.raises(ModelError):
            visualizer.score(self.binary.X.test, self.binary.y.test)

    def test_multi_decision_function_rocauc(self):
        """
        Test ROCAUC with multiclass classifiers that have a decision function
        """
        # Load the model and assert there is no predict_proba method.
        model = LinearSVC()
        with pytest.raises(AttributeError):
            model.predict_proba

        # Fit model and visualizer
        visualizer = ROCAUC(model)
        visualizer.fit(self.multiclass.X.train, self.multiclass.y.train)

        # First 5 expected arrays in the y_scores
        first_five_expected = [
            [-0.370, -0.543, -1.059, -0.466, -0.743, -1.156],
            [-0.445, -0.693, -0.362, -1.002, -0.815, -0.878],
            [-1.058, -0.808, -0.291, -0.767, -0.651, -0.586],
            [-0.446, -1.255, -0.489, -0.961, -0.807, -0.126],
            [-1.066, -0.493, -0.639, -0.442, -0.639, -1.017],
        ]

        # Get the predict_proba scores and evaluate
        y_scores = visualizer._get_y_scores(self.multiclass.X.train)

        # Check to see if the first 5 y_score arrays match the expected
        npt.assert_array_almost_equal(y_scores[:5], first_five_expected, decimal=1)

    def test_predict_proba_rocauc(self):
        """
        Test ROCAUC with classifiers that utilize predict_proba
        """
        # Load the model and assert there is no decision_function method.
        model = GaussianNB()
        with pytest.raises(AttributeError):
            model.decision_function

        # Fit model and visualizer
        visualizer = ROCAUC(model)
        visualizer.fit(self.binary.X.train, self.binary.y.train)

        # First 10 expected arrays in the y_scores
        first_ten_expected = np.asarray(
            [
                [0.595, 0.405],
                [0.161, 0.839],
                [0.990, 0.010],
                [0.833, 0.167],
                [0.766, 0.234],
                [0.996, 0.004],
                [0.592, 0.408],
                [0.007, 0.993],
                [0.035, 0.965],
                [0.764, 0.236],
            ]
        )

        # Get the predict_proba scores and evaluate
        y_scores = visualizer._get_y_scores(self.binary.X.train)

        # Check to see if the first 10 y_score arrays match the expected
        npt.assert_array_almost_equal(y_scores[:10], first_ten_expected, decimal=1)

    def test_no_scoring_function(self):
        """
        Test ROCAUC with classifiers that have no scoring method
        """
        visualizer = ROCAUC(FakeClassifier())
        with pytest.raises(ModelError):
            visualizer._get_y_scores(self.binary.X.train)

    def test_with_fitted(self):
        """
        Test that visualizer properly handles an already-fitted model
        """
        X, y = load_occupancy(return_dataset=True).to_numpy()

        model = GaussianNB().fit(X, y)
        classes = ["unoccupied", "occupied"]

        with patch.object(model, "fit") as mockfit:
            oz = ROCAUC(model, classes=classes)
            oz.fit(X, y)
            mockfit.assert_not_called()

        with patch.object(model, "fit") as mockfit:
            oz = ROCAUC(model, classes=classes, is_fitted=True)
            oz.fit(X, y)
            mockfit.assert_not_called()

        with patch.object(model, "fit") as mockfit:
            oz = ROCAUC(model, classes=classes, is_fitted=False)
            oz.fit(X, y)
            mockfit.assert_called_once_with(X, y)

    def test_binary_meta_param(self):
        """
        Test the binary meta param with ROCAUC
        """
        oz = ROCAUC(GaussianNB(), binary=False)
        assert oz.micro is True
        assert oz.macro is True
        assert oz.per_class is True

        oz = ROCAUC(GaussianNB(), binary=True)
        assert oz.micro is False
        assert oz.macro is False
        assert oz.per_class is False

    def test_within_pipeline(self):
        """
        Test that visualizer can be accessed within a sklearn pipeline
        """
        X, y = load_occupancy(return_dataset=True).to_pandas()
        classes = ["unoccupied", "occupied"]

        X_train, X_test, y_train, y_test = tts(
            X, y, test_size=0.2, shuffle=True, random_state=42
        )

        model = Pipeline(
            [
                ("minmax", MinMaxScaler()),
                ("matrix", ROCAUC(SVC(random_state=42), classes=classes, binary=True)),
            ]
        )

        model.fit(X_train, y_train)
        model.score(X_test, y_test)
        model["matrix"].finalize()
        self.assert_images_similar(model["matrix"], tol=12)

    def test_within_pipeline_quickmethod(self):
        """
        Test that visualizer quickmethod can be accessed within a
        sklearn pipeline
        """
        X, y = load_occupancy(return_dataset=True).to_pandas()

        X_train, X_test, y_train, y_test = tts(
            X, y, test_size=0.2, shuffle=True, random_state=42
        )

        model = Pipeline(
            [
                ("minmax", MinMaxScaler()),
                (
                    "matrix",
                    roc_auc(
                        SVC(random_state=42),
                        X_train,
                        y_train,
                        X_test,
                        y_test,
                        classes=["vacant", "occupied"],
                        show=False,
                        binary=True,
                    ),
                ),
            ]
        )
        self.assert_images_similar(model["matrix"], tol=12)

    def test_pipeline_as_model_input(self):
        """
        Test that visualizer can handle sklearn pipeline as model input
        """
        X, y = load_occupancy(return_dataset=True).to_pandas()
        classes = ["unoccupied", "occupied"]

        X_train, X_test, y_train, y_test = tts(
            X, y, test_size=0.2, shuffle=True, random_state=42
        )

        model = Pipeline([("minmax", MinMaxScaler()), ("svc", SVC(random_state=42))])

        oz = ROCAUC(model, classes=classes, binary=True)
        oz.fit(X_train, y_train)
        oz.score(X_test, y_test)
        oz.finalize()
        self.assert_images_similar(oz, tol=12)

    def test_pipeline_as_model_input_quickmethod(self):
        """
        Test that visualizer can handle sklearn pipeline as model input
        within a quickmethod
        """
        X, y = load_occupancy(return_dataset=True).to_pandas()

        X_train, X_test, y_train, y_test = tts(
            X, y, test_size=0.2, shuffle=True, random_state=42
        )

        model = Pipeline([("minmax", MinMaxScaler()), ("svc", SVC(random_state=42))])

        oz = roc_auc(
            model,
            X_train,
            y_train,
            X_test,
            y_test,
            classes=["vacant", "occupied"],
            show=False,
            binary=True,
        )
        self.assert_images_similar(oz, tol=12)



# tests.test_classifier.conftest
# Provides fixtures for the classification tests module.
#
# Author:  Benjamin Bengfort
# Created: Fri Mar 23 18:07:00 2018 -0400
#
# Copyright (C) 2018 The scikit-yb developers
# For license information, see LICENSE.txt
#
# ID: conftest.py [1e04216] benjamin@bengfort.com $

"""
Provides fixtures for the classification tests module.
"""

##########################################################################
## Imports
##########################################################################

import pytest

from tests.fixtures import Dataset, Split

from yellowbrick.exceptions import NotFitted
from sklearn.exceptions import NotFittedError
from sklearn.datasets import make_classification
from sklearn.model_selection import train_test_split as tts


##########################################################################
## Assertion Helpers
##########################################################################

ATTRS = ["classes_", "class_count_", "score_"]


def assert_not_fitted(estimator, attrs=ATTRS, X_test=None):
    """
    Check that the estimator is not fitted by ensuring it does not have
    any of the attributes specified in attrs. If X_test is specified,
    it is passed to predict, which must also raise a NotFitted exception.
    """
    __traceback_hide__ = True
    for attr in attrs:
        msg = "model is fitted, has {} attribute".format(attr)
        assert not hasattr(estimator, attr), msg

    if X_test is not None:
        with pytest.raises((NotFitted, NotFittedError)):
            estimator.predict(X_test)


def assert_fitted(estimator, attrs=ATTRS, X_test=None):
    """
    Check that the estimator is fitted by ensuring it does have the attributes
    passed in attrs. If X_test is specified, it is passed to predict which
    must not raise a NotFitted exception.
    """
    __traceback_hide__ = True
    for attr in attrs:
        msg = "model is not fitted, does not have {} attribute".format(attr)
        assert hasattr(estimator, attr), msg

    if X_test is not None:
        try:
            estimator.predict(X_test)
        except (NotFitted, NotFittedError):
            pytest.fail("estimator not fitted raised from predict")


##########################################################################
## Fixtures
##########################################################################


@pytest.fixture(scope="class")
def binary(request):
    """
    Creates a random binary classification dataset fixture
    """
    X, y = make_classification(
        n_samples=500,
        n_features=20,
        n_informative=8,
        n_redundant=2,
        n_classes=2,
        n_clusters_per_class=3,
        random_state=87,
    )

    X_train, X_test, y_train, y_test = tts(X, y, test_size=0.2, random_state=93)

    dataset = Dataset(Split(X_train, X_test), Split(y_train, y_test))
    request.cls.binary = dataset


@pytest.fixture(scope="class")
def multiclass(request):
    """
    Creates a random multiclass classification dataset fixture
    """
    X, y = make_classification(
        n_samples=500,
        n_features=20,
        n_informative=8,
        n_redundant=2,
        n_classes=6,
        n_clusters_per_class=3,
        random_state=87,
    )

    X_train, X_test, y_train, y_test = tts(X, y, test_size=0.2, random_state=93)

    dataset = Dataset(Split(X_train, X_test), Split(y_train, y_test))
    request.cls.multiclass = dataset



# tests.test_utils.test_target
# Tests for the target helper functions module.
#
# Author:   Benjamin Bengfort <benjamin@bengfort.com>
# Created:  Thu Dec 27 20:43:31 2018 -0500
#
# For license information, see LICENSE.txt
#
# ID: test_target.py [899c88a] benjamin@bengfort.com $

"""
Tests for the target helper functions module.
"""

##########################################################################
## Imports
##########################################################################

import pytest
import numpy as np

from yellowbrick.utils.target import *
from yellowbrick.exceptions import YellowbrickValueError
from sklearn.datasets import make_regression, make_classification


##########################################################################
## Target Color Type Tests
##########################################################################


@pytest.mark.parametrize(
    "value,expected",
    [
        (None, TargetType.SINGLE),
        (np.ones(15), TargetType.SINGLE),
        (["a", "b", "a", "b", "c"], TargetType.DISCRETE),
        ([1, 2, 1, 2, 3], TargetType.DISCRETE),
        ([0.23, 0.94, 1.3, -1.02, 0.11], TargetType.CONTINUOUS),
        ([1, 2, 0.2, 0.5, 1], TargetType.CONTINUOUS),
        (np.array([0.2, 2.2, 1.2, -3.1]), TargetType.CONTINUOUS),
        (np.array([[1, 2], [0, 2], [2, 1]]), TargetType.DISCRETE),
        (np.array([[[1, 2], [1, 2]], [[1, 2], [1, 2]]]), TargetType.UNKNOWN),
    ],
    ids=[
        "none",
        "ones",
        "list str",
        "list int",
        "list float",
        "mixed list",
        "float array",
        "multioutput",
        "cube",
    ],
)
def test_target_color_type(value, expected):
    """
    Test the target_color_type helper function with a variety of data types
    """
    assert target_color_type(value) == expected


@pytest.mark.parametrize(
    "n_classes,expected",
    [
        (2, TargetType.DISCRETE),
        (4, TargetType.DISCRETE),
        (MAX_DISCRETE_CLASSES, TargetType.DISCRETE),
        (MAX_DISCRETE_CLASSES + 3, TargetType.CONTINUOUS),
    ],
    ids=["binary", "multiclass", "max discrete", "too many discrete"],
)
def test_binary_target_color_type(n_classes, expected):
    """
    Test classification target color type
    """
    _, y = make_classification(n_classes=n_classes, n_informative=n_classes + 2)
    assert target_color_type(y) == expected


def test_regression_target_color_type():
    """
    Test regression target color type
    """
    _, y = make_regression()
    assert target_color_type(y) == TargetType.CONTINUOUS


@pytest.mark.parametrize(
    "val",
    [
        "auto",
        "single",
        "discrete",
        "continuous",
        "unknown",
        TargetType.AUTO,
        TargetType.SINGLE,
        TargetType.DISCRETE,
        TargetType.CONTINUOUS,
        TargetType.UNKNOWN,
    ],
)
def test_target_type_validate_valid(val):
    try:
        TargetType.validate(val)
    except YellowbrickValueError:
        pyetst.fail("valid target type raised validation error")


@pytest.mark.parametrize(
    "val", ["foo", 1, 3.14, "s", "DISCRETE", "CONTINUOUS", ["a", "b", "c"]]
)
def test_target_type_validate_invalid(val):
    with pytest.raises(YellowbrickValueError, match="unknown target color type"):
        TargetType.validate(val)


@pytest.mark.parametrize(
    "val,expected",
    [
        ("discrete", True),
        (TargetType.DISCRETE, True),
        ("DISCRETE", True),
        (8, False),
        ("FOO", False),
        (3.14, False),
        ("foo", False),
        (["discrete"], False),
        ({"discrete"}, False),
    ],
)
def test_target_type_equals(val, expected):
    assert (TargetType.DISCRETE == val) is expected
